using System.Linq;
using System.Collections.Generic;
using DataContracts;
namespace Ato.EN.IntegrationServices.CodeGenerationIDS {
    public partial class IDS2019Validator {
    
    #region Cross Form Rules - not supported yet

        #region VR.ATO.GEN.402009

        /*  VR.ATO.GEN.402009
                    Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^TFN <> ^ParentTFN

        Data Elements:
            
                    ^ParentTFN = ParentTFN
            
                    ^TFN = RP
                    */
        protected void VRATOGEN402009(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.402009
            
            assertion = false; // = ((report.RPExists == true) && (report.RPIdentifierTFN != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.402009", Severity = ProcessMessageSeverity.Error,
                    Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                    LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.402009

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSecuritisationOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTechnicalOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRoyaltiesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRevenueNoPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTangibleRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherServicesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherReinsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSoftwareAndITServicesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRevenueNonMonetaryPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSalesAndMarketingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLendingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestmentRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsInterestFreeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTotalNetRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTradingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpBranchOperationsInternallyRecordedDealingsContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLeasingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedTangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHedgingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBorrowingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTreasuryRelatedOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpTrustExemptionCodeContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOverseasIntangiblesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAdvisoryOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestmentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRentLeaseOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPReinsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRechargeAmountsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPResearchAndDevelopmentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGuaranteeTypeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralDeductionOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLendingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPEligibleContractOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDivision770OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDerivativesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPQuarterlyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAssetManagementOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCapitalNoPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAdvisoryRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPs128FAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDebtFactoringOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCapitalNonMonetaryPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLicenceFeesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLogisticsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPManagementAndAdminOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherInterestOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherInsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedIntangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherGuaranteesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPs128FOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsInterestBearingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsInterestFreeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCSPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsInterestBearingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHybridInstrumentsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPForeignHubsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCrossBorderHybridOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpRestructuringEventContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGuaranteeTypeRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000______________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPEligibleContractRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_______________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBorrowingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHedgingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDTAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000__________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTotalNetOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000___________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000____________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLeasingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438000

        /*  VR.ATO.GEN.438000
                    Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^StartDate <> ^ParentStartDate

        Data Elements:
            
                    ^ParentStartDate = ParentStartDate
            
                    ^StartDate = StartDate
                    */
        protected void VRATOGEN438000_____________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438000
            
            assertion = false; // = ( != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438000", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period start date is incorrect.",
                    LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTradingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438000

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Quarterly
                    */
        protected void VRATOGEN438001(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPQuarterlyExists == true) && (report.RPQuarterlyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPQuarterlyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.AssetManagement
                    */
        protected void VRATOGEN438001_(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPAssetManagementExists == true) && (report.RPAssetManagementEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAssetManagementOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.BranchOperations
                    */
        protected void VRATOGEN438001__(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPBranchOperationsExists == true) && (report.RPBranchOperationsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CapitalNonMonetaryPayment
                    */
        protected void VRATOGEN438001___(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCapitalNonMonetaryPaymentExists == true) && (report.RPCapitalNonMonetaryPaymentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCapitalNonMonetaryPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CapitalNoPayment
                    */
        protected void VRATOGEN438001____(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCapitalNoPaymentExists == true) && (report.RPCapitalNoPaymentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCapitalNoPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.DebtFactoring
                    */
        protected void VRATOGEN438001_____(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPDebtFactoringExists == true) && (report.RPDebtFactoringEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDebtFactoringOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Derivatives
                    */
        protected void VRATOGEN438001______(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPDerivativesExists == true) && (report.RPDerivativesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDerivativesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Insurance
                    */
        protected void VRATOGEN438001_______(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPInsuranceExists == true) && (report.RPInsuranceEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.LicenceFees
                    */
        protected void VRATOGEN438001________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLicenceFeesExists == true) && (report.RPLicenceFeesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLicenceFeesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Logistics
                    */
        protected void VRATOGEN438001_________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLogisticsExists == true) && (report.RPLogisticsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLogisticsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.ManagementAndAdmin
                    */
        protected void VRATOGEN438001__________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPManagementAndAdminExists == true) && (report.RPManagementAndAdminEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPManagementAndAdminOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherFinancialDealings
                    */
        protected void VRATOGEN438001___________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherFinancialDealingsExists == true) && (report.RPOtherFinancialDealingsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherGuarantees
                    */
        protected void VRATOGEN438001____________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherGuaranteesExists == true) && (report.RPOtherGuaranteesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherGuaranteesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherInsurance
                    */
        protected void VRATOGEN438001_____________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherInsuranceExists == true) && (report.RPOtherInsuranceEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherInsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherInterest
                    */
        protected void VRATOGEN438001______________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherInterestExists == true) && (report.RPOtherInterestEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherInterestOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedIntangibleNonRevenue
                    */
        protected void VRATOGEN438001_______________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedIntangibleNonRevenueExists == true) && (report.RPOtherNotReportedIntangibleNonRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedIntangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedRevenue
                    */
        protected void VRATOGEN438001________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedRevenueExists == true) && (report.RPOtherNotReportedRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedTangibleNonRevenue
                    */
        protected void VRATOGEN438001_________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedTangibleNonRevenueExists == true) && (report.RPOtherNotReportedTangibleNonRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedTangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherReinsurance
                    */
        protected void VRATOGEN438001__________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherReinsuranceExists == true) && (report.RPOtherReinsuranceEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherReinsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherServices
                    */
        protected void VRATOGEN438001___________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherServicesExists == true) && (report.RPOtherServicesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherServicesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.RechargeAmounts
                    */
        protected void VRATOGEN438001____________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPRechargeAmountsExists == true) && (report.RPRechargeAmountsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRechargeAmountsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Reinsurance
                    */
        protected void VRATOGEN438001_____________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPReinsuranceExists == true) && (report.RPReinsuranceEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPReinsuranceOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.RentLease
                    */
        protected void VRATOGEN438001______________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPRentLeaseExists == true) && (report.RPRentLeaseEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRentLeaseOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.ResearchAndDevelopment
                    */
        protected void VRATOGEN438001_______________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPResearchAndDevelopmentExists == true) && (report.RPResearchAndDevelopmentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPResearchAndDevelopmentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.RevenueNonMonetaryPayment
                    */
        protected void VRATOGEN438001________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPRevenueNonMonetaryPaymentExists == true) && (report.RPRevenueNonMonetaryPaymentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRevenueNonMonetaryPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.RevenueNoPayment
                    */
        protected void VRATOGEN438001_________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPRevenueNoPaymentExists == true) && (report.RPRevenueNoPaymentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRevenueNoPaymentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Royalties
                    */
        protected void VRATOGEN438001__________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPRoyaltiesExists == true) && (report.RPRoyaltiesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPRoyaltiesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.SalesAndMarketing
                    */
        protected void VRATOGEN438001___________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPSalesAndMarketingExists == true) && (report.RPSalesAndMarketingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSalesAndMarketingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Securitisation
                    */
        protected void VRATOGEN438001____________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPSecuritisationExists == true) && (report.RPSecuritisationEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSecuritisationOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.SoftwareAndITServices
                    */
        protected void VRATOGEN438001_____________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPSoftwareAndITServicesExists == true) && (report.RPSoftwareAndITServicesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSoftwareAndITServicesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.TangibleRevenue
                    */
        protected void VRATOGEN438001______________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTangibleRevenueExists == true) && (report.RPTangibleRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTangibleRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Technical
                    */
        protected void VRATOGEN438001_______________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTechnicalExists == true) && (report.RPTechnicalEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTechnicalOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.TreasuryRelated
                    */
        protected void VRATOGEN438001________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTreasuryRelatedExists == true) && (report.RPTreasuryRelatedEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTreasuryRelatedOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedAssignmentOfIntangibleNonRevenue
                    */
        protected void VRATOGEN438001_________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueExists == true) && (report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfIntangibleNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenue
                    */
        protected void VRATOGEN438001__________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueExists == true) && (report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfSharesOrEquityInterestsNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OtherNotReportedAssignmentOfLoansOrDebtsNonRevenue
                    */
        protected void VRATOGEN438001___________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueExists == true) && (report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherNotReportedAssignmentOfLoansOrDebtsNonRevenueOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.OverseasIntangibles
                    */
        protected void VRATOGEN438001____________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOverseasIntangiblesExists == true) && (report.RPOverseasIntangiblesEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOverseasIntangiblesOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Advisory
                    */
        protected void VRATOGEN438001_____________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPAdvisoryExists == true) && (report.RPAdvisoryEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAdvisoryOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.{TrustExemptionCode}
                    */
        protected void VRATOGEN438001______________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = (rpTrustExemptionCodeContext.EndDate != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpTrustExemptionCodeContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Division770
                    */
        protected void VRATOGEN438001_______________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPDivision770Exists == true) && (report.RPDivision770EndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDivision770OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.EligibleContract
                    */
        protected void VRATOGEN438001________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPEligibleContractExists == true) && (report.RPEligibleContractEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPEligibleContractOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.GuaranteeType
                    */
        protected void VRATOGEN438001_________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPGuaranteeTypeExists == true) && (report.RPGuaranteeTypeEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGuaranteeTypeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.GeneralDeduction
                    */
        protected void VRATOGEN438001__________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPGeneralDeductionExists == true) && (report.RPGeneralDeductionEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralDeductionOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Hedging
                    */
        protected void VRATOGEN438001___________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPHedgingExists == true) && (report.RPHedgingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHedgingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Investment
                    */
        protected void VRATOGEN438001____________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPInvestmentExists == true) && (report.RPInvestmentEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestmentOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Lending
                    */
        protected void VRATOGEN438001_____________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLendingExists == true) && (report.RPLendingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLendingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Other
                    */
        protected void VRATOGEN438001______________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherExists == true) && (report.RPOtherEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.TotalNet
                    */
        protected void VRATOGEN438001_______________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTotalNetExists == true) && (report.RPTotalNetEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTotalNetOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Trading
                    */
        protected void VRATOGEN438001________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTradingExists == true) && (report.RPTradingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTradingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Leasing
                    */
        protected void VRATOGEN438001_________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLeasingExists == true) && (report.RPLeasingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLeasingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CFCLC
                    */
        protected void VRATOGEN438001__________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCFCLCExists == true) && (report.RPCFCLCEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CFCSP
                    */
        protected void VRATOGEN438001___________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCFCSPExists == true) && (report.RPCFCSPEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCSPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CFCUC
                    */
        protected void VRATOGEN438001____________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCFCUCExists == true) && (report.RPCFCUCEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.{RestructuringEvent}
                    */
        protected void VRATOGEN438001_____________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = (rpRestructuringEventContext.EndDate != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpRestructuringEventContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.DTA
                    */
        protected void VRATOGEN438001______________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPDTAExists == true) && (report.RPDTAEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPDTAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.s128F
                    */
        protected void VRATOGEN438001_______________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPs128FExists == true) && (report.RPs128FEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPs128FOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.s128FA
                    */
        protected void VRATOGEN438001________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPs128FAExists == true) && (report.RPs128FAEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPs128FAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Advisory:RP.Advisory.RelatedParty
                    */
        protected void VRATOGEN438001_________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPAdvisoryRelatedPartyExists == true) && (report.RPAdvisoryRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPAdvisoryRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.BranchOperations:RP.BranchOperations.InterestBearing
                    */
        protected void VRATOGEN438001__________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPBranchOperationsInterestBearingExists == true) && (report.RPBranchOperationsInterestBearingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsInterestBearingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.EligibleContract:RP.EligibleContract.RelatedParty
                    */
        protected void VRATOGEN438001___________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPEligibleContractRelatedPartyExists == true) && (report.RPEligibleContractRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPEligibleContractRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Investment:RP.Investment.RelatedParty
                    */
        protected void VRATOGEN438001____________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPInvestmentRelatedPartyExists == true) && (report.RPInvestmentRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestmentRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Hedging:RP.Hedging.RelatedParty
                    */
        protected void VRATOGEN438001_____________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPHedgingRelatedPartyExists == true) && (report.RPHedgingRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHedgingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Borrowing:RP.Borrowing.RelatedParty
                    */
        protected void VRATOGEN438001______________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPBorrowingRelatedPartyExists == true) && (report.RPBorrowingRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBorrowingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.TotalNet:RP.TotalNet.RelatedParty
                    */
        protected void VRATOGEN438001_______________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTotalNetRelatedPartyExists == true) && (report.RPTotalNetRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTotalNetRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Other:RP.Other.RelatedParty
                    */
        protected void VRATOGEN438001________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherRelatedPartyExists == true) && (report.RPOtherRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.Borrowing
                    */
        protected void VRATOGEN438001_________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPBorrowingExists == true) && (report.RPBorrowingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBorrowingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Trading:RP.Trading.RelatedParty
                    */
        protected void VRATOGEN438001__________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPTradingRelatedPartyExists == true) && (report.RPTradingRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTradingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Lending:RP.Lending.RelatedParty
                    */
        protected void VRATOGEN438001___________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLendingRelatedPartyExists == true) && (report.RPLendingRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLendingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.GuaranteeType:RP.GuaranteeType.RelatedParty
                    */
        protected void VRATOGEN438001____________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPGuaranteeTypeRelatedPartyExists == true) && (report.RPGuaranteeTypeRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGuaranteeTypeRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.BranchOperations:RP.BranchOperations.InterestFree
                    */
        protected void VRATOGEN438001_____________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPBranchOperationsInterestFreeExists == true) && (report.RPBranchOperationsInterestFreeEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPBranchOperationsInterestFreeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.OtherFinancialDealings:RP.OtherFinancialDealings.InterestBearing
                    */
        protected void VRATOGEN438001______________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherFinancialDealingsInterestBearingExists == true) && (report.RPOtherFinancialDealingsInterestBearingEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsInterestBearingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.OtherFinancialDealings:RP.OtherFinancialDealings.InterestFree
                    */
        protected void VRATOGEN438001_______________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPOtherFinancialDealingsInterestFreeExists == true) && (report.RPOtherFinancialDealingsInterestFreeEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOtherFinancialDealingsInterestFreeOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.BranchOperations:RP.BranchOperations.{InternallyRecordedDealings}
                    */
        protected void VRATOGEN438001________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = (rpBranchOperationsInternallyRecordedDealingsContext.EndDate != );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(rpBranchOperationsInternallyRecordedDealingsContext.OccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.HybridInstruments
                    */
        protected void VRATOGEN438001_________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPHybridInstrumentsExists == true) && (report.RPHybridInstrumentsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPHybridInstrumentsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.ForeignHubs
                    */
        protected void VRATOGEN438001__________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPForeignHubsExists == true) && (report.RPForeignHubsEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPForeignHubsOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP.Leasing:RP.Leasing.RelatedParty
                    */
        protected void VRATOGEN438001___________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPLeasingRelatedPartyExists == true) && (report.RPLeasingRelatedPartyEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLeasingRelatedPartyOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP
                    */
        protected void VRATOGEN438001____________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPExists == true) && (report.RPEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001

        #region VR.ATO.GEN.438001

        /*  VR.ATO.GEN.438001
                    Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                    IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                       RETURN VALIDATION MESSAGE
                    ENDIF

            Technical Business Rule Format:
                    ^EndDate <> ^ParentEndDate

        Data Elements:
            
                    ^ParentEndDate = ParentEndDate
            
                    ^EndDate = RP:RP.CrossBorderHybrid
                    */
        protected void VRATOGEN438001_____________________________________________________________________________(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            //TODO: Write manual rule for VR.ATO.GEN.438001
            
            assertion = false; // = ((report.RPCrossBorderHybridExists == true) && (report.RPCrossBorderHybridEndDate != ));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438001", Severity = ProcessMessageSeverity.Error,
                    Description = @"The context period end date is incorrect.",
                    LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                    //Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPCrossBorderHybridOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.GEN.438001
        #endregion Cross Form Rules - not supported yet
        #region Manual Rules - Rules marked for manual coding

        #region VR.ATO.IDS.440003

        /*  VR.ATO.IDS.440003
                    If the foreign country codes have been completed then the response to 'Did you have dealings with international related parties, apart from dealings with related parties located in specified countries?' must be 'Yes'.

                Legacy Rule Format:
                    IF [IDS2] = FALSE AND COUNT(TUPLE(ids.0006.lodge.req.xx.xx:ForeignCountries)) <> 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [IDS2] = IDS:RP:rvctc4.02.05:InternationalDealings.RelatedPartiesTransactions.Indicator

            Technical Business Rule Format:
                    ^IDS2 = FALSE AND Count(^ForeignCountries) <> 0

        Data Elements:
            
                    RP:^IDS2 = tns:InternationalDealings.RelatedPartiesTransactions.Indicator
            
                    ^ForeignCountries = tns:ForeignCountries
                    */
        protected void VRATOIDS440003(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion  =   (report.IDS2 == false && Count(report.ForeignCountriesCollectionCount) != 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440003",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International related party dealings question incorrect",
                    LongDescription = @"If the foreign country codes have been completed then the response to 'Did you have dealings with international related parties, apart from dealings with related parties located in specified countries?' must be 'True'",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RelatedPartiesTransactions.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IDS.440003" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IDS2", Value = GetValueOrEmpty(report.IDS2) });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440003
        
        #region VR.ATO.IDS.440004

        /*  VR.ATO.IDS.440004
                    If there is a 'Yes' response to 'Did you have dealings with international related parties, apart from dealings with related parties located in specified countries?' then a foreign country code must be completed.

                Legacy Rule Format:
                    IF [IDS2] = TRUE AND COUNT(TUPLE(ids.0006.lodge.req.xx.xx:ForeignCountries)) = 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS2] = IDS:RP:rvctc4.02.05:InternationalDealings.RelatedPartiesTransactions.Indicator

            Technical Business Rule Format:
                    ^IDS2 = TRUE AND Count(^ForeignCountries) = 0

        Data Elements:
            
                    RP:^IDS2 = tns:InternationalDealings.RelatedPartiesTransactions.Indicator
            
                    ^ForeignCountries = tns:ForeignCountries
                    */
        protected void VRATOIDS440004(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion  =  report.IDS2 == true && Count(report.ForeignCountriesCollectionCount) == 0;
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440004",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Foreign country code required",
                    LongDescription = @"If there is a 'True' response to 'Did you have dealings with international related parties, apart from dealings with related parties located in specified countries?' then a foreign country code must be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RelatedPartiesTransactions.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IDS.440004" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IDS2", Value = GetValueOrEmpty(report.IDS2) });


                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440004
        
        #region VR.ATO.IDS.440005

        /*  VR.ATO.IDS.440005
                    If an expenditure or revenue amount is completed at 'Did you have dealing with international related parties, apart from the dealings with related parties located in specified countries?' or 'Did you have dealings with international related parties located in specified countries?', there must be at least one expenditure or revenue amount greater than zero at one of Questions 5, 6, 7, 8, 9, 10, 11 or 12.

                Legacy Rule Format:
                    IF ANY OCCURRENCE OF ([IDS5] > 0 OR [IDS6] > 0 OR [IDS12] > 0 OR [IDS13] > 0) AND (([IDS17] = NULL OR [IDS17] =  0) AND ([IDS18] = NULL OR [IDS18] =  0) AND ([IDS22] = NULL OR [IDS22] =  0) AND ([IDS23] = NULL OR [IDS23] =  0) AND ([IDS26] = NULL OR [IDS26] =  0) AND ([IDS27] = NULL OR [IDS27] =  0) AND ([IDS31] = NULL OR [IDS31] =  0) AND ([IDS32] = NULL OR [IDS32] =  0) AND ([IDS36] = NULL OR [IDS36] =  0) AND ([IDS37] = NULL OR [IDS37] =  0) AND ([IDS40] = NULL OR [IDS40] =  0) AND ([IDS41] = NULL OR [IDS41] =  0) AND ([IDS44] = NULL OR [IDS44] =  0) AND ([IDS45] = NULL OR [IDS45] =  0) AND ([IDS48] = NULL OR [IDS48] =  0) AND ([IDS49] = NULL OR [IDS49] =  0) AND ([IDS52] = NULL OR [IDS52] =  0) AND ([IDS53] = NULL OR [IDS53] =  0) AND ([IDS56] = NULL OR [IDS56] =  0) AND ([IDS57] = NULL OR [IDS57] =  0) AND ([IDS60] = NULL OR [IDS60] =  0) AND ([IDS61] = NULL OR [IDS61] =  0) AND ([IDS64] = NULL OR [IDS64] =  0) AND ([IDS65] = NULL OR [IDS65] =  0) AND ([IDS68] = NULL OR [IDS68] =  0) AND ([IDS69] = NULL OR [IDS69] =  0) AND ([IDS72] = NULL OR [IDS72] =  0) AND ([IDS73] = NULL OR [IDS73] =  0) AND ([IDS76] = NULL OR [IDS76] =  0) AND ([IDS77] = NULL OR [IDS77] =  0) AND ([IDS82] = NULL OR [IDS82] =  0) AND ([IDS83] = NULL OR [IDS83] =  0) AND ([IDS89] = NULL OR [IDS89] =  0) AND ([IDS90] = NULL OR [IDS90] =  0) AND ([IDS93] = NULL OR [IDS93] =  0) AND ([IDS94] = NULL OR [IDS94] =  0) AND ([IDS102] = NULL OR [IDS102] =  0) AND ([IDS103] = NULL OR [IDS103] =  0) AND ([IDS106] = NULL OR [IDS106] =  0) AND ([IDS107] = NULL OR [IDS107] =  0) AND ([IDS110] = NULL OR [IDS110] =  0) AND ([IDS111] = NULL OR [IDS111] =  0) AND ([IDS114] = NULL OR [IDS114] =  0) AND ([IDS115] = NULL OR [IDS115] =  0) AND ([IDS118] = NULL OR [IDS118] =  0) AND ([IDS119] = NULL OR [IDS119] =  0) AND ([IDS124] = NULL OR [IDS124] =  0) AND ([IDS125] = NULL OR [IDS125] =  0))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS5] = IDS:RP:bafpr1.02.16:Expense.Total.Amount IN TUPLE(ids.0006.lodge.req.02.00:ActivityCode) IN TUPLE(ids.0006.lodge.req.02.00:ForeignCountries)
                    [IDS6] = IDS:RP:bafpr1.02.26:Revenue.Total.Amount IN TUPLE(ids.0006.lodge.req.02.00:ActivityCode) IN TUPLE(ids.0006.lodge.req.02.00:ForeignCountries)
                    [IDS12] = IDS:RP:bafpr1.02.16:Expense.Total.Amount IN TUPLE(ids.0006.lodge.req.02.00:ActivityCode) IN TUPLE(ids.0006.lodge.req.02.00:SpecifiedCountries)
                    [IDS13] = IDS:RP:bafpr1.02.26:Revenue.Total.Amount IN TUPLE(ids.0006.lodge.req.02.00:ActivityCode) IN TUPLE(ids.0006.lodge.req.02.00:SpecifiedCountries)
                    [IDS17] = IDS:RP.TangibleRevenue:bafpr1.02.16:Expense.Total.Amount
                    [IDS18] = IDS:RP.TangibleRevenue:bafpr1.02.26:Revenue.Total.Amount
                    [IDS22] = IDS:RP.Royalties:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [IDS23] = IDS:RP.Royalties:bafpr1.02.12:Income.IncludedInAssessableTotal.Amount
                    [IDS26] = IDS:RP.LicenceFees:bafpr1.02.12:Expense.DeductionsTotal.Amount
                    [IDS27] = IDS:RP.LicenceFees:bafpr1.02.12:Income.IncludedInAssessableTotal.Amount
                    [IDS31] = IDS:RP.RentLease:bafpr1.02.16:Expense.Total.Amount
                    [IDS32] = IDS:RP.RentLease:bafpr1.02.26:Revenue.Total.Amount
                    [IDS36] = IDS:RP.TreasuryRelated:bafpr1.02.16:Expense.Total.Amount
                    [IDS37] = IDS:RP.TreasuryRelated:bafpr1.02.26:Revenue.Total.Amount
                    [IDS40] = IDS:RP.ManagementAndAdmin:bafpr1.02.16:Expense.Total.Amount
                    [IDS41] = IDS:RP.ManagementAndAdmin:bafpr1.02.26:Revenue.Total.Amount
                    [IDS44] = IDS:RP.Insurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS45] = IDS:RP.Insurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS48] = IDS:RP.Reinsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS49] = IDS:RP.Reinsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS52] = IDS:RP.ResearchAndDevelopment:bafpr1.02.16:Expense.Total.Amount
                    [IDS53] = IDS:RP.ResearchAndDevelopment:bafpr1.02.26:Revenue.Total.Amount
                    [IDS56] = IDS:RP.SalesAndMarketing:bafpr1.02.16:Expense.Total.Amount
                    [IDS57] = IDS:RP.SalesAndMarketing:bafpr1.02.26:Revenue.Total.Amount
                    [IDS60] = IDS:RP.SoftwareAndITServices:bafpr1.02.16:Expense.Total.Amount
                    [IDS61] = IDS:RP.SoftwareAndITServices:bafpr1.02.26:Revenue.Total.Amount
                    [IDS64] = IDS:RP.Technical:bafpr1.02.16:Expense.Total.Amount
                    [IDS65] = IDS:RP.Technical:bafpr1.02.26:Revenue.Total.Amount
                    [IDS68] = IDS:RP.Logistics:bafpr1.02.16:Expense.Total.Amount
                    [IDS69] = IDS:RP.Logistics:bafpr1.02.26:Revenue.Total.Amount
                    [IDS72] = IDS:RP.AssetManagement:bafpr1.02.16:Expense.Total.Amount
                    [IDS73] = IDS:RP.AssetManagement:bafpr1.02.26:Revenue.Total.Amount
                    [IDS76] = IDS:RP.OtherServices:bafpr1.02.16:Expense.Total.Amount
                    [IDS77] = IDS:RP.OtherServices:bafpr1.02.26:Revenue.Total.Amount
                    [IDS82] = IDS:RP.Derivatives:bafpr1.02.16:Expense.Total.Amount
                    [IDS83] = IDS:RP.Derivatives:bafpr1.02.26:Revenue.Total.Amount
                    [IDS89] = IDS:RP.DebtFactoring:bafpr1.02.12:Expense.BookValue.Amount
                    [IDS90] = IDS:RP.DebtFactoring:bafpr1.02.12:Expense.Consideration.Amount
                    [IDS93] = IDS:RP.Securitisation:bafpr1.02.12:Expense.BookValue.Amount
                    [IDS94] = IDS:RP.Securitisation:bafpr1.02.12:Expense.Consideration.Amount
                    [IDS102] = IDS:RP.OtherInterest:bafpr1.02.16:Expense.Total.Amount
                    [IDS103] = IDS:RP.OtherInterest:bafpr1.02.26:Revenue.Total.Amount
                    [IDS106] = IDS:RP.OtherGuarantees:bafpr1.02.16:Expense.Total.Amount
                    [IDS107] = IDS:RP.OtherGuarantees:bafpr1.02.26:Revenue.Total.Amount
                    [IDS110] = IDS:RP.OtherInsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS111] = IDS:RP.OtherInsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS114] = IDS:RP.OtherReinsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS115] = IDS:RP.OtherReinsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS118] = IDS:RP.OtherFinancialDealings:bafpr1.02.16:Expense.Total.Amount
                    [IDS119] = IDS:RP.OtherFinancialDealings:bafpr1.02.26:Revenue.Total.Amount
                    [IDS124] = IDS:RP.OtherNotReportedRevenue:bafpr1.02.16:Expense.Total.Amount
                    [IDS125] = IDS:RP.OtherNotReportedRevenue:bafpr1.02.26:Revenue.Total.Amount

            Technical Business Rule Format:
                    (AnyOccurrence(^IDS5, ^IDS5 > 0) OR ^IDS6 > 0 OR ^IDS12 > 0 OR ^IDS13 > 0) AND ((^IDS17 = NULL OR ^IDS17 =  0) AND (^IDS18 = NULL OR ^IDS18 =  0) AND (^IDS22 = NULL OR ^IDS22 =  0) AND (^IDS23 = NULL OR ^IDS23 =  0) AND (^IDS26 = NULL OR ^IDS26 =  0) AND (^IDS27 = NULL OR ^IDS27 =  0) AND (^IDS31 = NULL OR ^IDS31 =  0) AND (^IDS32 = NULL OR ^IDS32 =  0) AND (^IDS36 = NULL OR ^IDS36 =  0) AND (^IDS37 = NULL OR ^IDS37 =  0) AND (^IDS40 = NULL OR ^IDS40 =  0) AND (^IDS41 = NULL OR ^IDS41 =  0) AND (^IDS44 = NULL OR ^IDS44 =  0) AND (^IDS45 = NULL OR ^IDS45 =  0) AND (^IDS48 = NULL OR ^IDS48 =  0) AND (^IDS49 = NULL OR ^IDS49 =  0) AND (^IDS52 = NULL OR ^IDS52 =  0) AND (^IDS53 = NULL OR ^IDS53 =  0) AND (^IDS56 = NULL OR ^IDS56 =  0) AND (^IDS57 = NULL OR ^IDS57 =  0) AND (^IDS60 = NULL OR ^IDS60 =  0) AND (^IDS61 = NULL OR ^IDS61 =  0) AND (^IDS64 = NULL OR ^IDS64 =  0) AND (^IDS65 = NULL OR ^IDS65 =  0) AND (^IDS68 = NULL OR ^IDS68 =  0) AND (^IDS69 = NULL OR ^IDS69 =  0) AND (^IDS72 = NULL OR ^IDS72 =  0) AND (^IDS73 = NULL OR ^IDS73 =  0) AND (^IDS76 = NULL OR ^IDS76 =  0) AND (^IDS77 = NULL OR ^IDS77 =  0) AND (^IDS82 = NULL OR ^IDS82 =  0) AND (^IDS83 = NULL OR ^IDS83 =  0) AND (^IDS89 = NULL OR ^IDS89 =  0) AND (^IDS90 = NULL OR ^IDS90 =  0) AND (^IDS93 = NULL OR ^IDS93 =  0) AND (^IDS94 = NULL OR ^IDS94 =  0) AND (^IDS102 = NULL OR ^IDS102 =  0) AND (^IDS103 = NULL OR ^IDS103 =  0) AND (^IDS106 = NULL OR ^IDS106 =  0) AND (^IDS107 = NULL OR ^IDS107 =  0) AND (^IDS110 = NULL OR ^IDS110 =  0) AND (^IDS111 = NULL OR ^IDS111 =  0) AND (^IDS114 = NULL OR ^IDS114 =  0) AND (^IDS115 = NULL OR ^IDS115 =  0) AND (^IDS118 = NULL OR ^IDS118 =  0) AND (^IDS119 = NULL OR ^IDS119 =  0) AND (^IDS124 = NULL OR ^IDS124 =  0) AND (^IDS125 = NULL OR ^IDS125 =  0))

        Data Elements:
            
                    RP:^IDS5 = tns:Expense.Total.Amount
            
                    RP:^IDS6 = tns:Revenue.Total.Amount
            
                    RP:^IDS12 = tns:Expense.Total.Amount
            
                    RP:^IDS13 = tns:Revenue.Total.Amount
            
                    RP.TangibleRevenue:^IDS17 = tns:Expense.Total.Amount
            
                    RP.TangibleRevenue:^IDS18 = tns:Revenue.Total.Amount
            
                    RP.Royalties:^IDS22 = tns:Expense.DeductionsTotal.Amount
            
                    RP.Royalties:^IDS23 = tns:Income.IncludedInAssessableTotal.Amount
            
                    RP.LicenceFees:^IDS26 = tns:Expense.DeductionsTotal.Amount
            
                    RP.LicenceFees:^IDS27 = tns:Income.IncludedInAssessableTotal.Amount
            
                    RP.RentLease:^IDS31 = tns:Expense.Total.Amount
            
                    RP.RentLease:^IDS32 = tns:Revenue.Total.Amount
            
                    RP.TreasuryRelated:^IDS36 = tns:Expense.Total.Amount
            
                    RP.TreasuryRelated:^IDS37 = tns:Revenue.Total.Amount
            
                    RP.ManagementAndAdmin:^IDS40 = tns:Expense.Total.Amount
            
                    RP.ManagementAndAdmin:^IDS41 = tns:Revenue.Total.Amount
            
                    RP.Insurance:^IDS44 = tns:Expense.Total.Amount
            
                    RP.Insurance:^IDS45 = tns:Revenue.Total.Amount
            
                    RP.Reinsurance:^IDS48 = tns:Expense.Total.Amount
            
                    RP.Reinsurance:^IDS49 = tns:Revenue.Total.Amount
            
                    RP.ResearchAndDevelopment:^IDS52 = tns:Expense.Total.Amount
            
                    RP.ResearchAndDevelopment:^IDS53 = tns:Revenue.Total.Amount
            
                    RP.SalesAndMarketing:^IDS56 = tns:Expense.Total.Amount
            
                    RP.SalesAndMarketing:^IDS57 = tns:Revenue.Total.Amount
            
                    RP.SoftwareAndITServices:^IDS60 = tns:Expense.Total.Amount
            
                    RP.SoftwareAndITServices:^IDS61 = tns:Revenue.Total.Amount
            
                    RP.Technical:^IDS64 = tns:Expense.Total.Amount
            
                    RP.Technical:^IDS65 = tns:Revenue.Total.Amount
            
                    RP.Logistics:^IDS68 = tns:Expense.Total.Amount
            
                    RP.Logistics:^IDS69 = tns:Revenue.Total.Amount
            
                    RP.AssetManagement:^IDS72 = tns:Expense.Total.Amount
            
                    RP.AssetManagement:^IDS73 = tns:Revenue.Total.Amount
            
                    RP.OtherServices:^IDS76 = tns:Expense.Total.Amount
            
                    RP.OtherServices:^IDS77 = tns:Revenue.Total.Amount
            
                    RP.Derivatives:^IDS82 = tns:Expense.Total.Amount
            
                    RP.Derivatives:^IDS83 = tns:Revenue.Total.Amount
            
                    RP.DebtFactoring:^IDS89 = tns:Expense.BookValue.Amount
            
                    RP.DebtFactoring:^IDS90 = tns:Expense.Consideration.Amount
            
                    RP.Securitisation:^IDS93 = tns:Expense.BookValue.Amount
            
                    RP.Securitisation:^IDS94 = tns:Expense.Consideration.Amount
            
                    RP.OtherInterest:^IDS102 = tns:Expense.Total.Amount
            
                    RP.OtherInterest:^IDS103 = tns:Revenue.Total.Amount
            
                    RP.OtherGuarantees:^IDS106 = tns:Expense.Total.Amount
            
                    RP.OtherGuarantees:^IDS107 = tns:Revenue.Total.Amount
            
                    RP.OtherInsurance:^IDS110 = tns:Expense.Total.Amount
            
                    RP.OtherInsurance:^IDS111 = tns:Revenue.Total.Amount
            
                    RP.OtherReinsurance:^IDS114 = tns:Expense.Total.Amount
            
                    RP.OtherReinsurance:^IDS115 = tns:Revenue.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS118 = tns:Expense.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS119 = tns:Revenue.Total.Amount
            
                    RP.OtherNotReportedRevenue:^IDS124 = tns:Expense.Total.Amount
            
                    RP.OtherNotReportedRevenue:^IDS125 = tns:Revenue.Total.Amount
                    */
        protected void VRATOIDS440005(List<ProcessMessageDocument> response, IDS2019 report)
        {
            decimal? ids5 = null;
            decimal? ids6 = null;
            decimal? ids12 = null;
            decimal? ids13 = null;

            if (report.ForeignCountriesCollection != null)
            {
                ids5 =  report.ForeignCountriesCollection.Where (i =>i.ForeignCountries_ActivityCodeCollection !=null).SelectMany(i => i.ForeignCountries_ActivityCodeCollection ).Where(i => i.IDS5.GetValueOrDefault() > 0).Select(i => i.IDS5).FirstOrDefault();
                ids6 = report.ForeignCountriesCollection.Where(i => i.ForeignCountries_ActivityCodeCollection != null).SelectMany(i => i.ForeignCountries_ActivityCodeCollection).Where(i => i.IDS5.GetValueOrDefault() > 0).Select(i => i.IDS5).FirstOrDefault();
            }
            if (report.SpecifiedCountriesCollection != null)
            {
                ids12 = report.SpecifiedCountriesCollection.Where(i => i.SpecifiedCountries_ActivityCodeCollection != null).SelectMany(i => i.SpecifiedCountries_ActivityCodeCollection).Where(i => i.IDS12.GetValueOrDefault() > 0).Select(i => i.IDS12).FirstOrDefault();
                ids13 = report.SpecifiedCountriesCollection.Where(i => i.SpecifiedCountries_ActivityCodeCollection != null).SelectMany(i => i.SpecifiedCountries_ActivityCodeCollection).Where(i => i.IDS13.GetValueOrDefault() > 0).Select(i => i.IDS13).FirstOrDefault();
            } 

            ProcessMessageDocument processMessage; 
            bool assertion  =   (((ids5.GetValueOrDefault()  > 0 ) || ids6.GetValueOrDefault() > 0 || ids12.GetValueOrDefault() > 0 || ids13.GetValueOrDefault() > 0) && ((report.IDS17 == null || report.IDS17 == 0) && (report.IDS18 == null || report.IDS18 == 0) && (report.IDS22 == null || report.IDS22 == 0) && (report.IDS23 == null || report.IDS23 == 0) && (report.IDS26 == null || report.IDS26 == 0) && (report.IDS27 == null || report.IDS27 == 0) && (report.IDS31 == null || report.IDS31 == 0) && (report.IDS32 == null || report.IDS32 == 0) && (report.IDS36 == null || report.IDS36 == 0) && (report.IDS37 == null || report.IDS37 == 0) && (report.IDS40 == null || report.IDS40 == 0) && (report.IDS41 == null || report.IDS41 == 0) && (report.IDS44 == null || report.IDS44 == 0) && (report.IDS45 == null || report.IDS45 == 0) && (report.IDS48 == null || report.IDS48 == 0) && (report.IDS49 == null || report.IDS49 == 0) && (report.IDS52 == null || report.IDS52 == 0) && (report.IDS53 == null || report.IDS53 == 0) && (report.IDS56 == null || report.IDS56 == 0) && (report.IDS57 == null || report.IDS57 == 0) && (report.IDS60 == null || report.IDS60 == 0) && (report.IDS61 == null || report.IDS61 == 0) && (report.IDS64 == null || report.IDS64 == 0) && (report.IDS65 == null || report.IDS65 == 0) && (report.IDS68 == null || report.IDS68 == 0) && (report.IDS69 == null || report.IDS69 == 0) && (report.IDS72 == null || report.IDS72 == 0) && (report.IDS73 == null || report.IDS73 == 0) && (report.IDS76 == null || report.IDS76 == 0) && (report.IDS77 == null || report.IDS77 == 0) && (report.IDS82 == null || report.IDS82 == 0) && (report.IDS83 == null || report.IDS83 == 0) && (report.IDS89 == null || report.IDS89 == 0) && (report.IDS90 == null || report.IDS90 == 0) && (report.IDS93 == null || report.IDS93 == 0) && (report.IDS94 == null || report.IDS94 == 0) && (report.IDS102 == null || report.IDS102 == 0) && (report.IDS103 == null || report.IDS103 == 0) && (report.IDS106 == null || report.IDS106 == 0) && (report.IDS107 == null || report.IDS107 == 0) && (report.IDS110 == null || report.IDS110 == 0) && (report.IDS111 == null || report.IDS111 == 0) && (report.IDS114 == null || report.IDS114 == 0) && (report.IDS115 == null || report.IDS115 == 0) && (report.IDS118 == null || report.IDS118 == 0) && (report.IDS119 == null || report.IDS119 == 0) && (report.IDS124 == null || report.IDS124 == 0) && (report.IDS125 == null || report.IDS125 == 0)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440005", Severity = ProcessMessageSeverity.Error,
                    Description = @"Expenditure and revenue information incomplete",
                    LongDescription = @"If an expenditure or revenue amount is completed 'Did you have dealings with international related parties?' or 'Did you have dealings with international related parties located in specified countries?', there must be at least one expenditure or revenue amount greater than zero at ''Did you have any international related party dealings involving tangible property of a revenue nature?' or 'Did you have any international related party dealings involving royalties or licence fees?' or 'Did you have any international related party dealings involving rent or leasing? or 'Did you have any service arrangements with international related parties?' or 'Did you have any derivative transactions with international related parties?' or 'Did you enter into any debt factoring or securitisation arrangements with international related parties? or 'Did you have any dealings of a financial nature other than those listed with international related parties?' or Did you have any other international related party dealings of a revenue nature not reported?'",
                    Location = "/xbrli:xbrl/tns:ForeignCountries/tns:ActivityCode" ,//+ OccurrenceIndex(activityCode.OccurrenceIndex) + "/tns:Expense.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440005"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS5", Value = "IDS5" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS6", Value = ids6.GetValueOrDefault().ToString () });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS12", Value = ids12.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS13", Value = ids13.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS17", Value = report.IDS17.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS18", Value = report.IDS18.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS22", Value = report.IDS22.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS23", Value = report.IDS23.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS26", Value = report.IDS26.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS27", Value = report.IDS27.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS31", Value = report.IDS31.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS32", Value = report.IDS32.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS36", Value = report.IDS36.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS37", Value = report.IDS37.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS40", Value = report.IDS40.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS41", Value = report.IDS41.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS44", Value = report.IDS44.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS45", Value = report.IDS45.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS48", Value = report.IDS48.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS49", Value = report.IDS49.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS52", Value = report.IDS52.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS53", Value = report.IDS53.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS56", Value = report.IDS56.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS57", Value = report.IDS57.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS60", Value = report.IDS60.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS61", Value = report.IDS61.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS64", Value = report.IDS64.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS65", Value = report.IDS65.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS68", Value = report.IDS68.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS69", Value = report.IDS69.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS72", Value = report.IDS72.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS73", Value = report.IDS73.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS76", Value = report.IDS76.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS77", Value = report.IDS77.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS82", Value = report.IDS82.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS83", Value = report.IDS83.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS89", Value = report.IDS89.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS90", Value = report.IDS90.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS93", Value = report.IDS93.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS94", Value = report.IDS94.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS102", Value = report.IDS102.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS103", Value = report.IDS103.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS106", Value = report.IDS106.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS107", Value = report.IDS107.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS110", Value = report.IDS110.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS111", Value = report.IDS111.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS114", Value = report.IDS114.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS115", Value = report.IDS115.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS118", Value = report.IDS118.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS119", Value = report.IDS119.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS124", Value = report.IDS124.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS125", Value = report.IDS125.GetValueOrDefault().ToString() });
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440005

        #region VR.ATO.IDS.440010

        /*  VR.ATO.IDS.440010
                    If the specified country codes have been completed then the response to 'Did you have dealings with international related parties located in specified countries?' must be 'Yes'.

                Legacy Rule Format:
                    IF [IDS9] = FALSE AND COUNT(TUPLE(ids.0006.lodge.req.xx.xx:SpecifiedCountries)) <> 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator

            Technical Business Rule Format:
                    ^IDS9 = FALSE AND Count(^SpecifiedCountries) <> 0

        Data Elements:
            
                    RP:^IDS9 = tns:InternationalDealings.SpecifiedCountriesTransactions.Indicator
            
                    ^SpecifiedCountries = tns:SpecifiedCountries
                    */
        protected void VRATOIDS440010(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion  =  (report.IDS9 == false && Count(report.SpecifiedCountriesCollectionCount) != 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"International related party dealings question incorrect",
                    LongDescription = @"If the specified country codes have been completed then the response to 'Did you have dealings with international related parties located in specified countries?' must be 'True'",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.SpecifiedCountriesTransactions.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IDS.440010" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IDS9", Value = GetValueOrEmpty(report.IDS9) });

                                
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440010

        #region VR.ATO.IDS.440011

        /*  VR.ATO.IDS.440011
                    If there is a 'Yes' response to 'Did you have dealings with international related parties located in specified countries?' then the specified country code must be completed.

                Legacy Rule Format:
                    IF [IDS9] = TRUE AND COUNT(TUPLE(ids.0006.lodge.req.xx.xx:SpecifiedCountries)) = 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    
                    [IDS9] = IDS:RP:rvctc4.02.05:InternationalDealings.SpecifiedCountriesTransactions.Indicator

            Technical Business Rule Format:
                    ^IDS9 = TRUE AND Count(^SpecifiedCountries) = 0

        Data Elements:
            
                    RP:^IDS9 = tns:InternationalDealings.SpecifiedCountriesTransactions.Indicator
            
                    ^SpecifiedCountries = tns:SpecifiedCountries
                    */
        protected void VRATOIDS440011(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;
            bool assertion  =   (report.IDS9 == true && Count(report.SpecifiedCountriesCollectionCount) == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440011",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Specified country code required",
                    LongDescription = @"If there is a 'True' response to 'Did you have dealings with international related parties located in specified countries?' then the specified country code must be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.SpecifiedCountriesTransactions.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IDS.440011" } },
                };

                //  Parameters are commented out or non-existant for manual rules 
                    processMessage.Parameters.Add(new ProcessMessageParameter
                        { Name = "IDS9", Value = GetValueOrEmpty(report.IDS9) });

                 
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440011
        
        #region VR.ATO.IDS.440045

        /*  VR.ATO.IDS.440045
                    If International related party dealings information has been completed for financial dealings for amounts borrowed, amounts loaned, interest, guarantees, insurance, reinsurance, foreign exchange or other financial dealings then the response to 'Did you have any dealings of a financial nature with international related parties?' must be 'True'.

                Legacy Rule Format:
                    IF [IDS97] <> TRUE AND ([IDS98] <> NULL OR [IDS99] <> NULL OR [IDS100] <> NULL OR [IDS101] <> NULL OR [IDS102] <> NULL OR [IDS103] <> NULL OR [IDS104] <> NULL OR [IDS105] <> NULL OR [IDS106] <> NULL OR [IDS107] <> NULL OR [IDS108] <> NULL OR [IDS109] <> NULL OR [IDS110] <>NULL OR [IDS111] <> NULL OR [IDS112] <> NULL OR [IDS113] <> NULL OR [IDS114] <> NULL OR [IDS115] <> NULL OR [IDS116] <> NULL OR [IDS117] <> NULL OR [IDS118] <> NULL OR [IDS119] <> NULL OR [IDS120] <> NULL OR [IDS121] <> NULL OR [IDS122] <> NULL OR [IDS333] <> NULL OR [IDS336] <> NULL OR [IDS337] <> NULL OR [IDS338] <> NULL OR [IDS339] <> NULL)
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS97] = IDS:RP:rvctc4.02.05:InternationalDealings.OtherFinancialDealings.Indicator
                    [IDS98] = IDS:RP.OtherFinancialDealings.InterestBearing:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS99] = IDS:RP.OtherFinancialDealings.InterestFree:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS100] = IDS:RP.OtherFinancialDealings.InterestBearing:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS101] = IDS:RP.OtherFinancialDealings.InterestFree:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS102] = IDS:RP.OtherInterest:bafpr1.02.16:Expense.Total.Amount
                    [IDS103] = IDS:RP.OtherInterest:bafpr1.02.26:Revenue.Total.Amount
                    [IDS104] = IDS:RP.OtherInterest:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS105] = IDS:RP.OtherInterest:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS106] = IDS:RP.OtherGuarantees:bafpr1.02.16:Expense.Total.Amount
                    [IDS107] = IDS:RP.OtherGuarantees:bafpr1.02.26:Revenue.Total.Amount
                    [IDS108] = IDS:RP.OtherGuarantees:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS109] = IDS:RP.OtherGuarantees:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS110] = IDS:RP.OtherInsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS111] = IDS:RP.OtherInsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS112] = IDS:RP.OtherInsurance:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS113] = IDS:RP.OtherInsurance:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS114] = IDS:RP.OtherReinsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS115] = IDS:RP.OtherReinsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS116] = IDS:RP.OtherReinsurance:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS117] = IDS:RP.OtherReinsurance:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS118] = IDS:RP.OtherFinancialDealings:bafpr1.02.16:Expense.Total.Amount
                    [IDS119] = IDS:RP.OtherFinancialDealings:bafpr1.02.26:Revenue.Total.Amount
                    [IDS120] = IDS:RP.OtherFinancialDealings:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS121] = IDS:RP.OtherFinancialDealings:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS122] = IDS:RP.OtherFinancialDealings:rvctc4.02.05:InternationalDealings.Description.Text
                    [IDS333] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainLoss.Indicator
                    [IDS336] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeLossesHighestNominal.Amount IN TUPLE(ids.0006.lodge.req.02.00:CurrencyCode) IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS337] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainsHighestNominal.Amount IN TUPLE(ids.0006.lodge.req.02.00:CurrencyCode) IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS338] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeLossesOther.Amount IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS339] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainsOther.Amount IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)

            Technical Business Rule Format:
                    ^IDS97 <> TRUE AND (^IDS98 <> NULL OR ^IDS99 <> NULL OR ^IDS100 <> NULL OR ^IDS101 <> NULL OR ^IDS102 <> NULL OR ^IDS103 <> NULL OR ^IDS104 <> NULL OR ^IDS105 <> NULL OR ^IDS106 <> NULL OR ^IDS107 <> NULL OR ^IDS108 <> NULL OR ^IDS109 <> NULL OR ^IDS110 <> NULL OR ^IDS111 <> NULL OR ^IDS112 <> NULL OR ^IDS113 <> NULL OR ^IDS114 <> NULL OR ^IDS115 <> NULL OR ^IDS116 <> NULL OR ^IDS117 <> NULL OR ^IDS118 <> NULL OR ^IDS119 <> NULL OR ^IDS120 <> NULL OR ^IDS121 <> NULL OR ^IDS122 <> NULL OR ^IDS333 <> NULL OR ^IDS336 <> NULL OR ^IDS337 <> NULL OR ^IDS338 <> NULL OR ^IDS339 <> NULL)

        Data Elements:
            
                    RP:^IDS97 = tns:InternationalDealings.OtherFinancialDealings.Indicator
            
                    RP.OtherFinancialDealings.InterestBearing:^IDS98 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestFree:^IDS99 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestBearing:^IDS100 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestFree:^IDS101 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.OtherInterest:^IDS102 = tns:Expense.Total.Amount
            
                    RP.OtherInterest:^IDS103 = tns:Revenue.Total.Amount
            
                    RP.OtherInterest:^IDS104 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherInterest:^IDS105 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherGuarantees:^IDS106 = tns:Expense.Total.Amount
            
                    RP.OtherGuarantees:^IDS107 = tns:Revenue.Total.Amount
            
                    RP.OtherGuarantees:^IDS108 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherGuarantees:^IDS109 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherInsurance:^IDS110 = tns:Expense.Total.Amount
            
                    RP.OtherInsurance:^IDS111 = tns:Revenue.Total.Amount
            
                    RP.OtherInsurance:^IDS112 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherInsurance:^IDS113 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherReinsurance:^IDS114 = tns:Expense.Total.Amount
            
                    RP.OtherReinsurance:^IDS115 = tns:Revenue.Total.Amount
            
                    RP.OtherReinsurance:^IDS116 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherReinsurance:^IDS117 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherFinancialDealings:^IDS118 = tns:Expense.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS119 = tns:Revenue.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS120 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherFinancialDealings:^IDS121 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherFinancialDealings:^IDS122 = tns:InternationalDealings.Description.Text
            
                    RP:^IDS333 = tns:InternationalDealings.ForeignExchangeGainLoss.Indicator
            
                    RP:^IDS336 = tns:InternationalDealings.ForeignExchangeLossesHighestNominal.Amount
            
                    RP:^IDS337 = tns:InternationalDealings.ForeignExchangeGainsHighestNominal.Amount
            
                    RP:^IDS338 = tns:InternationalDealings.ForeignExchangeLossesOther.Amount
            
                    RP:^IDS339 = tns:InternationalDealings.ForeignExchangeGainsOther.Amount
                    */
        protected void VRATOIDS440045(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;

            decimal? ids336 = null;
            decimal? ids337 = null;
            decimal? ids338 = null;
            decimal? ids339 = null;

            if (report.TransactionTypeCollection != null)
            {
                ids338 = report.TransactionTypeCollection.Where(i => i.IDS338 != null).Select(i => i.IDS338).FirstOrDefault();
                ids339 = report.TransactionTypeCollection.Where(i => i.IDS339 != null).Select(i => i.IDS339).FirstOrDefault();

                ids336 = report.TransactionTypeCollection.Where(i => i.TransactionType_CurrencyCodeCollection != null).SelectMany(i => i.TransactionType_CurrencyCodeCollection).Where(i => i.IDS336 != null).Select(i => i.IDS336).FirstOrDefault();
                ids337 = report.TransactionTypeCollection.Where(i => i.TransactionType_CurrencyCodeCollection != null).SelectMany(i => i.TransactionType_CurrencyCodeCollection).Where(i => i.IDS337 != null).Select(i => i.IDS337).FirstOrDefault(); 
            } 
            bool assertion = report.IDS97 != true &&
                             (report.IDS98 != null || report.IDS99 != null || report.IDS100 != null ||
                              report.IDS101 != null || report.IDS102 != null || report.IDS103 != null ||
                              report.IDS104 != null || report.IDS105 != null || report.IDS106 != null ||
                              report.IDS107 != null || report.IDS108 != null || report.IDS109 != null ||
                              report.IDS110 != null || report.IDS111 != null || report.IDS112 != null ||
                              report.IDS113 != null || report.IDS114 != null || report.IDS115 != null ||
                              report.IDS116 != null || report.IDS117 != null || report.IDS118 != null ||
                              report.IDS119 != null || report.IDS120 != null || report.IDS121 != null ||
                              report.IDS122 != null || report.IDS333 != null || ids336 != null ||
                              ids337 != null || ids338 != null || ids339 != null);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440455", Severity = ProcessMessageSeverity.Error,
                    Description = @"International related party dealings question incorrect",
                    LongDescription = @"If International related party dealings information has been completed for financial dealings for amounts borrowed, amounts loaned, interest, guarantees, insurance, reinsurance, foreign exchange or other financial dealings then the response to 'Did you have any dealings of a financial nature with international related parties?' must be 'True'",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.OtherFinancialDealings.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440045"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS97", Value = GetValueOrEmpty(report.IDS97) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS98", Value = report.IDS98.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS99", Value = report.IDS99.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS100", Value = report.IDS100.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS101", Value = report.IDS101.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS102", Value = report.IDS102.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS103", Value = report.IDS103.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS104", Value = report.IDS104 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS105", Value = report.IDS105 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS106", Value = report.IDS106.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS107", Value = report.IDS107.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS108", Value = report.IDS108 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS109", Value = report.IDS109 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS110", Value = report.IDS110.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS111", Value = report.IDS111.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS112", Value = report.IDS112 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS113", Value = report.IDS113 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS114", Value = report.IDS114.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS115", Value = report.IDS115.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS116", Value = report.IDS116 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS117", Value = report.IDS117 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS118", Value = report.IDS118.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS119", Value = report.IDS119.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS120", Value = report.IDS120 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS121", Value = report.IDS121 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS122", Value = report.IDS122 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS333", Value = GetValueOrEmpty(report.IDS333) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS336", Value =  ids336.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS337", Value = ids337.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS338", Value = ids338.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS339", Value = ids339.GetValueOrDefault().ToString() });
    
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440045

        #region VR.ATO.IDS.440046

        /*  VR.ATO.IDS.440046
                    If securitisation information has been completed then book value or consideration, main pricing methodology code and percentage of dealings with documentation code must be completed

                Legacy Rule Format:
                    IF ([IDS97] = TRUE AND [IDS98] = NULL AND [IDS99] = NULL AND [IDS100] = NULL AND [IDS101] = NULL AND [IDS102] = NULL AND [IDS103] = NULL AND [IDS104] = NULL AND [IDS105] = NULL AND [IDS106] = NULL AND [IDS107] = NULL AND [IDS108] = NULL AND [IDS109] = NULL AND [IDS110] = NULL AND [IDS111] = NULL AND [IDS112] = NULL AND [IDS113] = NULL AND [IDS114] = NULL AND [IDS115] = NULL AND [IDS116] = NULL AND [IDS117] = NULL AND [IDS118] = NULL AND [IDS119] = NULL AND [IDS120] = NULL AND [IDS121] = NULL AND [IDS122] = NULL AND [IDS333] = NULL AND [IDS336] = NULL AND [IDS337] = NULL AND [IDS338] = NULL AND [IDS339] = NULL)
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS97] = IDS:RP:rvctc4.02.05:InternationalDealings.OtherFinancialDealings.Indicator
                    [IDS98] = IDS:RP.OtherFinancialDealings.InterestBearing:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS99] = IDS:RP.OtherFinancialDealings.InterestFree:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS100] = IDS:RP.OtherFinancialDealings.InterestBearing:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS101] = IDS:RP.OtherFinancialDealings.InterestFree:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS102] = IDS:RP.OtherInterest:bafpr1.02.16:Expense.Total.Amount
                    [IDS103] = IDS:RP.OtherInterest:bafpr1.02.26:Revenue.Total.Amount
                    [IDS104] = IDS:RP.OtherInterest:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS105] = IDS:RP.OtherInterest:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS106] = IDS:RP.OtherGuarantees:bafpr1.02.16:Expense.Total.Amount
                    [IDS107] = IDS:RP.OtherGuarantees:bafpr1.02.26:Revenue.Total.Amount
                    [IDS108] = IDS:RP.OtherGuarantees:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS109] = IDS:RP.OtherGuarantees:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS110] = IDS:RP.OtherInsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS111] = IDS:RP.OtherInsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS112] = IDS:RP.OtherInsurance:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS113] = IDS:RP.OtherInsurance:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS114] = IDS:RP.OtherReinsurance:bafpr1.02.16:Expense.Total.Amount
                    [IDS115] = IDS:RP.OtherReinsurance:bafpr1.02.26:Revenue.Total.Amount
                    [IDS116] = IDS:RP.OtherReinsurance:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS117] = IDS:RP.OtherReinsurance:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS118] = IDS:RP.OtherFinancialDealings:bafpr1.02.16:Expense.Total.Amount
                    [IDS119] = IDS:RP.OtherFinancialDealings:bafpr1.02.26:Revenue.Total.Amount
                    [IDS120] = IDS:RP.OtherFinancialDealings:bafot.02.11:Miscellaneous.ArmsLengthPricingMethod.Code
                    [IDS121] = IDS:RP.OtherFinancialDealings:bafot.02.23:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
                    [IDS122] = IDS:RP.OtherFinancialDealings:rvctc4.02.05:InternationalDealings.Description.Text
                    [IDS333] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainLoss.Indicator
                    [IDS336] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeLossesHighestNominal.Amount IN TUPLE(ids.0006.lodge.req.02.00:CurrencyCode) IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS337] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainsHighestNominal.Amount IN TUPLE(ids.0006.lodge.req.02.00:CurrencyCode) IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS338] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeLossesOther.Amount IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)
                    [IDS339] = IDS:RP:rvctc4.02.13:InternationalDealings.ForeignExchangeGainsOther.Amount IN TUPLE(ids.0006.lodge.req.02.00:TransactionType)

            Technical Business Rule Format:
                    ^IDS97 = TRUE AND ^IDS98 = NULL AND ^IDS99 = NULL AND ^IDS100 = NULL AND ^IDS101 = NULL AND ^IDS102 = NULL AND ^IDS103 = NULL AND ^IDS104 = NULL AND ^IDS105 = NULL AND ^IDS106 = NULL AND ^IDS107 = NULL AND ^IDS108 = NULL AND ^IDS109 = NULL AND ^IDS110 = NULL AND ^IDS111 = NULL AND ^IDS112 = NULL AND ^IDS113 = NULL AND ^IDS114 = NULL AND ^IDS115 = NULL AND ^IDS116 = NULL AND ^IDS117 = NULL AND ^IDS118 = NULL AND ^IDS119 = NULL AND ^IDS120 = NULL AND ^IDS121 = NULL AND ^IDS122 = NULL AND ^IDS333 = NULL AND AnyOccurrence(^IDS336, ^IDS336 = NULL) AND AnyOccurrence(^IDS337, ^IDS337 = NULL) AND AnyOccurrence(^IDS338, ^IDS338 = NULL) AND AnyOccurrence(^IDS339, ^IDS339 = NULL)

        Data Elements:
            
                    RP:^IDS97 = tns:InternationalDealings.OtherFinancialDealings.Indicator
            
                    RP.OtherFinancialDealings.InterestBearing:^IDS98 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestFree:^IDS99 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestBearing:^IDS100 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.OtherFinancialDealings.InterestFree:^IDS101 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.OtherInterest:^IDS102 = tns:Expense.Total.Amount
            
                    RP.OtherInterest:^IDS103 = tns:Revenue.Total.Amount
            
                    RP.OtherInterest:^IDS104 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherInterest:^IDS105 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherGuarantees:^IDS106 = tns:Expense.Total.Amount
            
                    RP.OtherGuarantees:^IDS107 = tns:Revenue.Total.Amount
            
                    RP.OtherGuarantees:^IDS108 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherGuarantees:^IDS109 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherInsurance:^IDS110 = tns:Expense.Total.Amount
            
                    RP.OtherInsurance:^IDS111 = tns:Revenue.Total.Amount
            
                    RP.OtherInsurance:^IDS112 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherInsurance:^IDS113 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherReinsurance:^IDS114 = tns:Expense.Total.Amount
            
                    RP.OtherReinsurance:^IDS115 = tns:Revenue.Total.Amount
            
                    RP.OtherReinsurance:^IDS116 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherReinsurance:^IDS117 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherFinancialDealings:^IDS118 = tns:Expense.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS119 = tns:Revenue.Total.Amount
            
                    RP.OtherFinancialDealings:^IDS120 = tns:Miscellaneous.ArmsLengthPricingMethod.Code
            
                    RP.OtherFinancialDealings:^IDS121 = tns:Miscellaneous.PercentageOfDealingsWithDocumentation.Code
            
                    RP.OtherFinancialDealings:^IDS122 = tns:InternationalDealings.Description.Text
            
                    RP:^IDS333 = tns:InternationalDealings.ForeignExchangeGainLoss.Indicator
            
                    RP:^IDS336 = tns:InternationalDealings.ForeignExchangeLossesHighestNominal.Amount
            
                    RP:^IDS337 = tns:InternationalDealings.ForeignExchangeGainsHighestNominal.Amount
            
                    RP:^IDS338 = tns:InternationalDealings.ForeignExchangeLossesOther.Amount
            
                    RP:^IDS339 = tns:InternationalDealings.ForeignExchangeGainsOther.Amount
                    */
        protected void VRATOIDS440046(List<ProcessMessageDocument> response, IDS2019 report)
        {
            decimal? ids336 = null;
            decimal? ids337 = null;
            decimal? ids338 = null;
            decimal? ids339 = null;

            if (report.TransactionTypeCollection != null)
            {
                ids338 = report.TransactionTypeCollection.Where(i => i.IDS338 != null).Select(i => i.IDS338).FirstOrDefault();
                ids339 = report.TransactionTypeCollection.Where(i => i.IDS339 != null).Select(i => i.IDS339).FirstOrDefault();

                ids336 = report.TransactionTypeCollection.Where(i => i.TransactionType_CurrencyCodeCollection != null).SelectMany(i => i.TransactionType_CurrencyCodeCollection).Where(i => i.IDS336 != null).Select(i => i.IDS336).FirstOrDefault();
                ids337 = report.TransactionTypeCollection.Where(i => i.TransactionType_CurrencyCodeCollection != null).SelectMany(r => r.TransactionType_CurrencyCodeCollection).Where(i => i.IDS337 != null).Select(i => i.IDS337).FirstOrDefault();
            }

             
            ProcessMessageDocument processMessage; 
            bool assertion =   (report.IDS97 == true && report.IDS98 == null && report.IDS99 == null && report.IDS100 == null && report.IDS101 == null && report.IDS102 == null && report.IDS103 == null && report.IDS104 == null && report.IDS105 == null && report.IDS106 == null && report.IDS107 == null && report.IDS108 == null && report.IDS109 == null && report.IDS110 == null && report.IDS111 == null && report.IDS112 == null && report.IDS113 == null && report.IDS114 == null && report.IDS115 == null && report.IDS116 == null && report.IDS117 == null && report.IDS118 == null && report.IDS119 == null && report.IDS120 == null && report.IDS121 == null && report.IDS122 == null && report.IDS333 == null && ids336 == null && ids337 == null && ids338 == null && ids339 == null );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440456", Severity = ProcessMessageSeverity.Error,
                    Description = @"International related party dealings information required",
                    LongDescription = @"If there is a 'Yes' response to 'Did you have any dealings of a financial nature with international related parties?' then financial dealings information for amounts borrowed, amounts loaned, interest, guarantees, insurance, reinsurance, foreign exchange or other financial dealings must be completed.",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.OtherFinancialDealings.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440046"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS97", Value = GetValueOrEmpty(report.IDS97) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS98", Value = report.IDS98.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS99", Value = report.IDS99.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS100", Value = report.IDS100.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS101", Value = report.IDS101.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS102", Value = report.IDS102.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS103", Value = report.IDS103.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS104", Value = report.IDS104 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS105", Value = report.IDS105 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS106", Value = report.IDS106.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS107", Value = report.IDS107.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS108", Value = report.IDS108 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS109", Value = report.IDS109 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS110", Value = report.IDS110.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS111", Value = report.IDS111.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS112", Value = report.IDS112 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS113", Value = report.IDS113 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS114", Value = report.IDS114.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS115", Value = report.IDS115.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS116", Value = report.IDS116 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS117", Value = report.IDS117 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS118", Value = report.IDS118.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS119", Value = report.IDS119.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS120", Value = report.IDS120 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS121", Value = report.IDS121 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS122", Value = report.IDS122 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS333", Value = GetValueOrEmpty(report.IDS333) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS336", Value = ids336.GetValueOrDefault ().ToString () });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS337", Value = ids337.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS338", Value = ids338.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS339", Value = ids339.GetValueOrDefault().ToString() });
    
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440046

        #region VR.ATO.IDS.440075

        /*  VR.ATO.IDS.440075
                    If there is a 'No' response to 'Did a restructuring event occur in the current year involving international related parties or your branch operations?' then restructuring event information should not be completed.

                Legacy Rule Format:
                    IF [IDS151] = FALSE AND COUNT(RP.{RestructuringEvent}:ANY ELEMENT <> NULL) > 0
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS151] = IDS:RP:rvctc4.02.05:InternationalDealings.RestructuringEvents.Indicator

            Technical Business Rule Format:
                    ^IDS151 = FALSE AND Count(^RestructuringEvent) > 0

        Data Elements:
            
                    RP:^IDS151 = tns:InternationalDealings.RestructuringEvents.Indicator
            
                    RP:^RestructuringEvent = RP:RP.{RestructuringEvent}
                    */
        protected void VRATOIDS440075(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;  
            bool assertion = report.IDS151 == false && report.RPRestructuringEventContextCollection != null &&
                            report.RPRestructuringEventContextCollection.Any(i => i.IDS152 != null || i.IDS153 != null
                               || i.IDS154 != null || i.IDS155 != null || i.IDS156 != null || i.IDS158 != null);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440075", Severity = ProcessMessageSeverity.Error,
                    Description = @"Branch operations restructuring event information is not required",
                    LongDescription = @"If there is a 'False' response to 'Did a restructuring event occur in the current year involving international related parties or your branch operations?' then restructuring event information should not be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RestructuringEvents.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440075"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS151", Value = GetValueOrEmpty(report.IDS151) });
    
    
             
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440075

        #region VR.ATO.IDS.440076

        /*  VR.ATO.IDS.440076
                    If there is a 'Yes' response to 'Did a restructuring event occur in the current year involving international related parties or your branch operations?' then restructuring event information must be completed.

                Legacy Rule Format:
                    IF [IDS151] = TRUE AND COUNT(SET(RP.{RestructuringEvent}):ANY ELEMENT <> NULL) = 0
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS151] = IDS:RP:rvctc4.02.05:InternationalDealings.RestructuringEvents.Indicator

            Technical Business Rule Format:
                    ^IDS151 = TRUE AND Count(^RestructuringEvent) = 0

        Data Elements:
            
                    RP:^IDS151 = tns:InternationalDealings.RestructuringEvents.Indicator
            
                    RP:^RestructuringEvent = RP:RP.{RestructuringEvent}
                    */
        protected void VRATOIDS440076(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;  
            bool assertion = report.IDS151 == true &&
              (report.RPRestructuringEventContextCollection == null
              || report.RPRestructuringEventContextCollection != null &&
                   !report.RPRestructuringEventContextCollection.Any(i => i.IDS152 != null
                                                                      || i.IDS153 != null
                                                                      || i.IDS154 != null
                                                                      || i.IDS155 != null
                                                                      || i.IDS156 != null
                                                                      || i.IDS158 != null));

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440076", Severity = ProcessMessageSeverity.Error,
                    Description = @"Branch operations restructuring event information is required",
                    LongDescription = @"If there is a 'True' response to 'Did a restructuring event occur in the current year involving international related parties or your branch operations?' then restructuring event information must be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RestructuringEvents.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440076"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS151", Value = GetValueOrEmpty(report.IDS151) });
     
            
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440076

        #region VR.ATO.IDS.440080

        /*  VR.ATO.IDS.440080
                    If there is a 'No' response to 'Do you have any branch operations (including Australian branch operations if you are a non-resident)?' then amounts claimed or returned for internally recorded dealings with branch operations should not be completed.

                Legacy Rule Format:
                    IF [IDS160] = FALSE AND ([IDS161] <> NULL AND [IDS162] <> NULL AND [IDS163] <> NULL AND [IDS164] <> NULL AND [IDS165] <> NULL AND [IDS166] <> NULL AND [IDS167] <> NULL AND [IDS168] <> NULL) AND COUNT(RP.BranchOperations.{InternallyRecordedDealings}:ANY ELEMENT <> NULL) > 0 
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS160] = IDS:RP:rvctc4.02.05:InternationalDealings.BranchOperations.Indicator
                    [IDS161] = IDS:RP.BranchOperations.InterestBearing:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS162] = IDS:RP.BranchOperations.InterestBearing:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS163] = IDS:RP.BranchOperations.InterestBearing:bafpr1.02.12:Expense.InterestBorrowedTotal.Amount
                    [IDS164] = IDS:RP.BranchOperations.InterestBearing:bafpr1.02.12:Income.InterestLoanedTotal.Amount
                    [IDS165] = IDS:RP.BranchOperations.InterestFree:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS166] = IDS:RP.BranchOperations.InterestFree:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS167] = IDS:RP.BranchOperations:bafpr1.02.12:Expense.TradingStockPurchaseCosts.Amount
                    [IDS168] = IDS:RP.BranchOperations:bafpr1.02.12:Income.TradingStockSalesTotal.Amount

            Technical Business Rule Format:
                    ^IDS160 = FALSE AND (^IDS161 <> NULL AND ^IDS162 <> NULL AND ^IDS163 <> NULL AND ^IDS164 <>  NULL AND ^IDS165 <> NULL AND ^IDS166 <> NULL AND ^IDS167 <> NULL AND ^IDS168 <> NULL) AND (Count(^InternallyRecordedDealings) > 0)

        Data Elements:
            
                    RP:^IDS160 = tns:InternationalDealings.BranchOperations.Indicator
            
                    RP.BranchOperations.InterestBearing:^IDS161 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS162 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS163 = tns:Expense.InterestBorrowedTotal.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS164 = tns:Income.InterestLoanedTotal.Amount
            
                    RP.BranchOperations.InterestFree:^IDS165 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.BranchOperations.InterestFree:^IDS166 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.BranchOperations:^IDS167 = tns:Expense.TradingStockPurchaseCosts.Amount
            
                    RP.BranchOperations:^IDS168 = tns:Income.TradingStockSalesTotal.Amount
            
                    RP.BranchOperations:^InternallyRecordedDealings = RP.BranchOperations:RP.BranchOperations.{InternallyRecordedDealings}
                    */
        protected void VRATOIDS440080(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion = report.IDS160 == false && (report.IDS161 != null || report.IDS162 != null ||
                            report.IDS163 != null || report.IDS164 != null || report.IDS165 != null ||
                            report.IDS166 != null || report.IDS167 != null || report.IDS168 != null ||
                            report.RPBranchOperationsInternallyRecordedDealingsContextCollection != null &&
                            report.RPBranchOperationsInternallyRecordedDealingsContextCollection.Any(
                                i => i.IDS169 != null || i.IDS170 != null));


            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440080", Severity = ProcessMessageSeverity.Error,
                    Description = @"Branch operations information is not required",
                    LongDescription = @"If there is a 'False' response to 'Do you have any branch operations (including Australian branch operations if you are a non-resident)?' then amounts claimed or returned for internally recorded dealings with branch operations should not be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.BranchOperations.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440080"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS160", Value = GetValueOrEmpty(report.IDS160) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS161", Value = report.IDS161.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS162", Value = report.IDS162.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS163", Value = report.IDS163.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS164", Value = report.IDS164.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS165", Value = report.IDS165.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS166", Value = report.IDS166.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS167", Value = report.IDS167.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS168", Value = report.IDS168.GetValueOrDefault().ToString() });
    
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440080

        #region VR.ATO.IDS.440081

        /*  VR.ATO.IDS.440081
                    If there is a 'Yes' response to 'Do you have any branch operations (including Australian branch operations if you are a non-resident)?' then amounts claimed or returned for internally recorded dealings with branch operations must be completed.

                Legacy Rule Format:
                    IF [IDS160] = TRUE AND ([IDS161] = NULL AND [IDS162] = NULL AND [IDS163] = NULL AND [IDS164] = NULL AND [IDS165] = NULL AND [IDS166] = NULL AND [IDS167] = NULL AND [IDS168] = NULL) AND COUNT (RP.BranchOperations.{InternallyRecordedDealings}:ANY ELEMENT <> NULL) = 0 
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS160] = IDS:RP:rvctc4.02.05:InternationalDealings.BranchOperations.Indicator
                    [IDS161] = IDS:RP.BranchOperations.InterestBearing:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS162] = IDS:RP.BranchOperations.InterestBearing:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS163] = IDS:RP.BranchOperations.InterestBearing:bafpr1.02.12:Expense.InterestBorrowedTotal.Amount
                    [IDS164] = IDS:RP.BranchOperations.InterestBearing:bafpr1.02.12:Income.InterestLoanedTotal.Amount
                    [IDS165] = IDS:RP.BranchOperations.InterestFree:bafpo1.02.11:Liabilities.BorrowedAverageBalance.Amount
                    [IDS166] = IDS:RP.BranchOperations.InterestFree:bafpo1.02.11:Assets.LoanedAverageBalance.Amount
                    [IDS167] = IDS:RP.BranchOperations:bafpr1.02.12:Expense.TradingStockPurchaseCosts.Amount
                    [IDS168] = IDS:RP.BranchOperations:bafpr1.02.12:Income.TradingStockSalesTotal.Amount

            Technical Business Rule Format:
                    ^IDS160 = TRUE AND (^IDS161 = NULL AND ^IDS162 = NULL AND ^IDS163 = NULL AND    ^IDS164 = NULL AND ^IDS165 = NULL AND ^IDS166 = NULL AND ^IDS167 = NULL AND ^IDS168 = NULL) AND (Count(^InternallyRecordedDealings) = 0)

        Data Elements:
            
                    RP:^IDS160 = tns:InternationalDealings.BranchOperations.Indicator
            
                    RP.BranchOperations.InterestBearing:^IDS161 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS162 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS163 = tns:Expense.InterestBorrowedTotal.Amount
            
                    RP.BranchOperations.InterestBearing:^IDS164 = tns:Income.InterestLoanedTotal.Amount
            
                    RP.BranchOperations.InterestFree:^IDS165 = tns:Liabilities.BorrowedAverageBalance.Amount
            
                    RP.BranchOperations.InterestFree:^IDS166 = tns:Assets.LoanedAverageBalance.Amount
            
                    RP.BranchOperations:^IDS167 = tns:Expense.TradingStockPurchaseCosts.Amount
            
                    RP.BranchOperations:^IDS168 = tns:Income.TradingStockSalesTotal.Amount
            
                    RP.BranchOperations:^InternallyRecordedDealings = RP.BranchOperations:RP.BranchOperations.{InternallyRecordedDealings}
                    */
        protected void VRATOIDS440081(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion = report.IDS160 == true && report.IDS161 == null && report.IDS162 == null &&
                            report.IDS163 == null && report.IDS164 == null && report.IDS165 == null &&
                            report.IDS166 == null && report.IDS167 == null && report.IDS168 == null &&
                            (report.RPBranchOperationsInternallyRecordedDealingsContextCollection == null
                            || report.RPBranchOperationsInternallyRecordedDealingsContextCollection != null &&
                            !report.RPBranchOperationsInternallyRecordedDealingsContextCollection.Any(
                                i => i.IDS169 != null || i.IDS170 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440081", Severity = ProcessMessageSeverity.Error,
                    Description = @"Branch operations information is required",
                    LongDescription = @"If there is a 'True' response to 'Do you have any branch operations (including Australian branch operations if you are a non-resident)?' then amounts claimed or returned for internally recorded dealings with branch operations must be completed",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.BranchOperations.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440081"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS160", Value = GetValueOrEmpty(report.IDS160) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS161", Value = report.IDS161.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS162", Value = report.IDS162.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS163", Value = report.IDS163.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS164", Value = report.IDS164.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS165", Value = report.IDS165.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS166", Value = report.IDS166.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS167", Value = report.IDS167.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS168", Value = report.IDS168.GetValueOrDefault().ToString() });
    
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440081

        #region VR.ATO.IDS.440296

        /*  VR.ATO.IDS.440296
                    If Interests in foreign entities information has been completed for transfers of property, money or services then the response to 'During the last three income years, including the current income year, did you directly or indirectly transfer property, money or services to a non-resident trust, where that non-resident trust was still in existence during that income year?' must be 'Yes'.

                Legacy Rule Format:
                    IF [IDS213] <> TRUE AND (COUNT(SET(RP.{TrustExemptionCode}):ANY ELEMENT <> NULL) > 0)
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS213] = IDS:RP:bafot.02.11:Miscellaneous.TransferPropertyMoneyOrServices.Indicator

            Technical Business Rule Format:
                    ^IDS213 <> TRUE AND Count(^Context) > 0

        Data Elements:
            
                    RP:^IDS213 = tns:Miscellaneous.TransferPropertyMoneyOrServices.Indicator
            
                    RP:^Context = RP:RP.{TrustExemptionCode}
                    */
        protected void VRATOIDS440296(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage;  
            bool assertion = report.IDS213 != true && report.RPTrustExemptionCodeContextCollection != null &&
                             report.RPTrustExemptionCodeContextCollection.Any(i => i.IDS214 != null);

            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440296", Severity = ProcessMessageSeverity.Error,
                    Description = @"Interests in foreign entities question incorrect",
                    LongDescription = @"If Interests in foreign entities information has been completed for transfers of property, money or services then the response to 'During the last three income years, including the current income year, did you directly or indirectly transfer property, money or services to a non-resident trust, where that non-resident trust was still in existence during that income year?' must be 'True'",
                    Location = "/xbrli:xbrl/tns:Miscellaneous.TransferPropertyMoneyOrServices.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440296"} },
                };
    
            //  Parameters are commented out or non-existant for manual rules 
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS213", Value = GetValueOrEmpty(report.IDS213) });
    
     
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440296

        #region VR.ATO.IDS.440425

        /*  VR.ATO.IDS.440425
                    In derivative transactions information the same principal derivative type code has been entered more than once. Each derivative code can only appear once.

                Legacy Rule Format:
                    IF [IDS332] = ANY OTHER OCCURRENCE OF [IDS332] 
                     RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS332] = IDS:RP.Derivatives:rvctc4.02.13:InternationalDealings.RelatedPartyTransactionType.Code

            Technical Business Rule Format:
                    HasDuplicateValues(^IDS332)

        Data Elements:
            
                    RP.Derivatives:^IDS332 = tns:InternationalDealings.RelatedPartyTransactionType.Code
                    */
        protected void VRATOIDS440425(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion = HasDuplicateValues(report.IDS332Collection);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440291", Severity = ProcessMessageSeverity.Error,
                    Description = @"Duplicate derivative code present for derivative transactions",
                    LongDescription = @"In derivative transactions information the same principal derivative type code has been entered more than once. Each derivative code can only appear once",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RelatedPartyTransactionType.Code" + OccurrenceIndex(DuplicateValueIndex(report.IDS332Collection) + 1) + "",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440425"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440425

        #region VR.ATO.IDS.440618

        /*  VR.ATO.IDS.440618
                    Gross revenue of CFCs that have satisfied the active income test - Listed countries must be completed, if there is a 'Yes' response to "CFCs satisfied the active income test - Listed countries".

                Legacy Rule Format:
                    IF [IDS443] = NULL AND [IDS375] = SET ("Yes","Both")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS375] = IDS:RP.CFCLC:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS443] = IDS:RP.CFCLC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount

            Technical Business Rule Format:
                    ^IDS443 = NULL AND InSet(^IDS375, '"Yes","Both"')

        Data Elements:
    
            ^IDS443 = RP.CFCLC:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
    
            ^IDS375 = RP.CFCLC:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440618(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            
            
            assertion = (report.IDS443 == null && report.IDS375 != null && IsMatch(report.IDS375.ToString(), @"^(Yes|Both)$"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440618", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have satisfied the active income test - Listed countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestSatisfied.Amount[@contextRef='" + report.RPCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440618"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS443", Value = GetValueOrEmpty(report.IDS443) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS375", Value = GetValueOrEmpty(report.IDS375) });
    
    
            
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440618

        #region VR.ATO.IDS.440619

        /*  VR.ATO.IDS.440619
                    Gross revenue of CFCs that have satisfied the active income test - Specified countries must be completed if there is a 'Yes' response to "CFCs satisfied the active income test - Specified countries".

                Legacy Rule Format:
                    IF [IDS444] = NULL AND [IDS376] = SET ("Yes","Both")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS376] = IDS:RP.CFCSP:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS444] = IDS:RP.CFCSP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount

            Technical Business Rule Format:
                    ^IDS444 = NULL AND InSet(^IDS376, '"Yes","Both"')

        Data Elements:
    
            ^IDS444 = RP.CFCSP:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
    
            ^IDS376 = RP.CFCSP:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440619(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

           
            
            assertion = (report.IDS444 == null && report.IDS376 != null &&  IsMatch(report.IDS376.ToString(), @"^(Yes|Both)$"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440619", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have satisfied the active income test - Specified countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestSatisfied.Amount[@contextRef='" + report.RPCFCSPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440619"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS444", Value = GetValueOrEmpty(report.IDS444) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS376", Value = GetValueOrEmpty(report.IDS376) });
    
    
            
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440619

         #region VR.ATO.IDS.440620
        /*  VR.ATO.IDS.440620
                    Gross revenue of CFCs that have satisfied the active income test - Other unlisted countries must be completed if there is a 'Yes' response to "CFCs satisfied the active income test - Other unlisted countries".

                Legacy Rule Format:
                    IF [IDS445] = NULL AND [IDS377] = SET ("Yes","Both")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS377] = IDS:RP.CFCUC:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS445] = IDS:RP.CFCUC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount

            Technical Business Rule Format:
                    ^IDS445 = NULL AND InSet(^IDS377, '"Yes","Both"')

        Data Elements:
    
            ^IDS445 = RP.CFCUC:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
    
            ^IDS377 = RP.CFCUC:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440620(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            
            
            assertion = (report.IDS445 == null && report.IDS377 != null && IsMatch(report.IDS377.ToString(), @"^(Yes|Both)$"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440620", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have satisfied the active income test - Other unlisted countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestSatisfied.Amount[@contextRef='" + report.RPCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440620"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS445", Value = GetValueOrEmpty(report.IDS445) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS377", Value = GetValueOrEmpty(report.IDS377) });
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440620


         #region VR.ATO.IDS.440621

        /*  VR.ATO.IDS.440621
                    Gross revenue of CFCs that have not satisfied the active income test - Listed countries must be completed if there is a 'No' response to "CFCs satisfied the active income test - Listed countries".

                Legacy Rule Format:
                    IF [IDS447] = NULL AND [IDS375] = SET ("No","Both")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS375] = IDS:RP.CFCLC:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS447] = IDS:RP.CFCLC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount

            Technical Business Rule Format:
                    ^IDS447 = NULL AND InSet(^IDS375, '"No","Both"')

        Data Elements:
    
            ^IDS447 = RP.CFCLC:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
    
            ^IDS375 = RP.CFCLC:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440621(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            
            assertion = (report.IDS447 == null && report.IDS375 != null && IsMatch(report.IDS375.ToString(), @"^(No|Both)$"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440621", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have not satisfied the active income test - Listed countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount[@contextRef='" + report.RPCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440621"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS447", Value = GetValueOrEmpty(report.IDS447) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS375", Value = GetValueOrEmpty(report.IDS375) });
    
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440621

         #region VR.ATO.IDS.440622

        /*  VR.ATO.IDS.440622
                    Gross revenue of CFCs that have not satisfied the active income test - Specified countries must be completed if there is a 'No' response to "CFCs satisfied the active income test - Specified countries".

                Legacy Rule Format:
                    IF [IDS376] = SET ("No","Both") AND [IDS448] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS376] = IDS:RP.CFCSP:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS448] = IDS:RP.CFCSP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount

            Technical Business Rule Format:
                    InSet(^IDS376, '"No","Both"') AND ^IDS448 = NULL

        Data Elements:
    
            ^IDS448 = RP.CFCSP:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
    
            ^IDS376 = RP.CFCSP:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440622(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            
            
            assertion = report.IDS376 != null && (IsMatch(report.IDS376.ToString(), @"^(No|Both)$") && report.IDS448 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440622", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have not satisfied the active income test - Specified countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount[@contextRef='" + report.RPCFCSPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440622"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS376", Value = GetValueOrEmpty(report.IDS376) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS448", Value = GetValueOrEmpty(report.IDS448) });
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440622
         #region VR.ATO.IDS.440623

        /*  VR.ATO.IDS.440623
                    Gross revenue of CFCs that have not satisfied the active income test - Other unlisted countries must be completed if there is a 'No' response to "CFCs satisfied the active income test - Other unlisted countries".

                Legacy Rule Format:
                    IF [IDS377] = SET ("No","Both") AND [IDS449] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS377] = IDS:RP.CFCUC:bafot.02.35:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
                    [IDS449] = IDS:RP.CFCUC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount

            Technical Business Rule Format:
                    InSet(^IDS377, '"No","Both"') AND ^IDS449 = NULL

        Data Elements:
    
            ^IDS449 = RP.CFCUC:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
    
            ^IDS377 = RP.CFCUC:Miscellaneous.ControlledForeignCompaniesActiveIncomeTestSatisfied.Code
            */
        protected void VRATOIDS440623(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

          
            
            assertion = report.IDS377 != null && (IsMatch(report.IDS377.ToString(), @"^(No|Both)$") && report.IDS449 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440623", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross revenue of CFCs that have not satisfied the active income test - Other unlisted countries is required.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount[@contextRef='" + report.RPCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440623"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules */
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS377", Value = GetValueOrEmpty(report.IDS377) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IDS449", Value = GetValueOrEmpty(report.IDS449) });
            
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440623

         #region VR.ATO.IDS.440583

        /*  VR.ATO.IDS.440583
                    Total amounts of excluded tainted interest income must be equal to the sum of amounts listed at tainted interest income excluded from passive income of your financial intermediary subsidiary CFCs for Listed, Specified and Other unlisted countries.

                Legacy Rule Format:
                    IF [IDS439] <> NULL AND ([IDS439] <> ([IDS436] + [IDS437] + [IDS438]) +/- 1)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS436] = IDS:RP.CFCLC:bafpr1.02.36:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
                    [IDS437] = IDS:RP.CFCSP:bafpr1.02.36:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
                    [IDS438] = IDS:RP.CFCUC:bafpr1.02.36:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
                    [IDS439] = IDS:RP:bafpr1.02.36:Income.ControlledForeignCompaniesExcludedTaintedInterestIncomeTotal.Amount

            Technical Business Rule Format:
                    ^IDS439 <> NULL AND (^IDS439 <> (^IDS436 + ^IDS437 + ^IDS438) +/- 1)

        Data Elements:
    
            ^IDS439 = RP:Income.ControlledForeignCompaniesExcludedTaintedInterestIncomeTotal.Amount
    
            ^IDS436 = RP.CFCLC:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
    
            ^IDS437 = RP.CFCSP:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
    
            ^IDS438 = RP.CFCUC:Income.ControlledForeignCompaniesExcludedTaintedInterestIncome.Amount
            */
        protected void VRATOIDS440583(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;
            assertion = (report.IDS439 != null && OutsideRange(report.IDS439.GetValueOrDefault(),
                             report.IDS436.GetValueOrDefault() + report.IDS437.GetValueOrDefault() +
                             report.IDS438.GetValueOrDefault(), 1));
                if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440583", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total excluded tainted interest income is not equal to the sum of, tainted interest income excluded from passive income of your financial intermediary subsidiary CFCs for Listed, Specified and Other unlisted countries.",
                    Location = "/xbrli:xbrl/tns:Income.ControlledForeignCompaniesExcludedTaintedInterestIncomeTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440583"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440583
        #region VR.ATO.IDS.440593
        /*  VR.ATO.IDS.440593
                    Total gross revenue included in the gross turnover of your CFCs that have satisfied the active income test, must be equal to the sum of amounts listed at the gross revenue for Listed, Specified and Other unlisted countries.

                Legacy Rule Format:
                    IF [IDS446] <> NULL AND ([IDS446] <> ([IDS443] + [IDS444] + [IDS445] +/- 1))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS443] = IDS:RP.CFCLC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
                    [IDS444] = IDS:RP.CFCSP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
                    [IDS445] = IDS:RP.CFCUC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
                    [IDS446] = IDS:RP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestSatisfiedTotal.Amount

            Technical Business Rule Format:
                    ^IDS446 <> NULL AND (^IDS446 <> (^IDS443 + ^IDS444 + ^IDS445 +/- 1))

        Data Elements:
    
            ^IDS446 = RP:Income.GrossRevenueActiveIncomeTestSatisfiedTotal.Amount
    
            ^IDS443 = RP.CFCLC:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
    
            ^IDS444 = RP.CFCSP:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
    
            ^IDS445 = RP.CFCUC:Income.GrossRevenueActiveIncomeTestSatisfied.Amount
            */
        protected void VRATOIDS440593(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;
            
            assertion = (report.IDS446 != null && OutsideRange(report.IDS446.GetValueOrDefault(),
                             report.IDS443.GetValueOrDefault() + report.IDS444.GetValueOrDefault() +
                             report.IDS445.GetValueOrDefault(), 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440593", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total gross revenue included in the gross turnover of your CFCs that have satisfied the active income test is not equal to the sum of amounts listed at the gross revenue for Listed, Specified and Other unlisted countries.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestSatisfiedTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440593"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440593

        #region VR.ATO.IDS.440594

        /*  VR.ATO.IDS.440594
                    Total gross revenue included in the gross turnover of your CFCs that have not satisfied the active income test, must be equal to the sum of amounts listed at the gross revenue for Listed, Specified and Other unlisted countries.

                Legacy Rule Format:
                    IF [IDS450] <> NULL AND ([IDS450] <> ([IDS447] + [IDS448] + [IDS449] +/- 1))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS447] = IDS:RP.CFCLC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
                    [IDS448] = IDS:RP.CFCSP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
                    [IDS449] = IDS:RP.CFCUC:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
                    [IDS450] = IDS:RP:bafpr1.02.36:Income.GrossRevenueActiveIncomeTestNotSatisfiedTotal.Amount

            Technical Business Rule Format:
                    ^IDS450 <> NULL AND (^IDS450 <> (^IDS447 + ^IDS448 + ^IDS449 +/- 1))

        Data Elements:
    
            ^IDS450 = RP:Income.GrossRevenueActiveIncomeTestNotSatisfiedTotal.Amount
    
            ^IDS447 = RP.CFCLC:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
    
            ^IDS448 = RP.CFCSP:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
    
            ^IDS449 = RP.CFCUC:Income.GrossRevenueActiveIncomeTestNotSatisfied.Amount
            */
        protected void VRATOIDS440594(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;

            assertion = (report.IDS450 != null && OutsideRange(report.IDS450.GetValueOrDefault(),
                                          report.IDS447.GetValueOrDefault() + report.IDS448.GetValueOrDefault() +
                                          report.IDS449.GetValueOrDefault(), 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440594", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total gross revenue included in the gross turnover of your CFCs that have not satisfied the active income test is not equal to the sum of amounts listed at the gross revenue for Listed, Specified and Other unlisted countries.",
                    Location = "/xbrli:xbrl/tns:Income.GrossRevenueActiveIncomeTestNotSatisfiedTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440594"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440594
        #region VR.ATO.IDS.440602
        /*  VR.ATO.IDS.440602
                    Total notional allowable deductions must be equal to the sum of amounts listed at notional allowable deductions for Listed countries, Specified countries and Other unlisted countries.

                Legacy Rule Format:
                    IF [IDS454] <> NULL AND ([IDS454] <> ([IDS451] + [IDS452] + [IDS453] +/- 1))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [IDS451] = IDS:RP.CFCLC:bafpr1.02.36:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
                    [IDS452] = IDS:RP.CFCSP:bafpr1.02.36:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
                    [IDS453] = IDS:RP.CFCUC:bafpr1.02.36:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
                    [IDS454] = IDS:RP:bafpr1.02.36:Income.ControlledForeignCompaniesNotionalAllowableDeductionsTotal.Amount

            Technical Business Rule Format:
                    ^IDS454 <> NULL AND (^IDS454 <> (^IDS451 + ^IDS452 + ^IDS453 +/- 1))

        Data Elements:
    
            ^IDS454 = RP:Income.ControlledForeignCompaniesNotionalAllowableDeductionsTotal.Amount
    
            ^IDS451 = RP.CFCLC:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
    
            ^IDS452 = RP.CFCSP:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
    
            ^IDS453 = RP.CFCUC:Income.ControlledForeignCompaniesNotionalAllowableDeductions.Amount
            */
        protected void VRATOIDS440602(List<ProcessMessageDocument> response, IDS2019 report)
        {
            ProcessMessageDocument processMessage; 
            bool assertion;
          
            assertion = (report.IDS454 != null && OutsideRange(report.IDS454.GetValueOrDefault(),
                             report.IDS451.GetValueOrDefault() + report.IDS452.GetValueOrDefault() +
                             report.IDS453.GetValueOrDefault(), 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IDS.440602", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total notional allowable deductions is not equal to the sum of amounts listed at notional allowable deductions for Listed countries, Specified countries and Other unlisted countries.",
                    Location = "/xbrli:xbrl/tns:Income.ControlledForeignCompaniesNotionalAllowableDeductionsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IDS.440602"} },
                };
    
            /*  Parameters are commented out or non-existant for manual rules 
    
            */
                response.Add(processMessage);
            }
        }
        #endregion // VR.ATO.IDS.440602

    #endregion Manual Rules - Rules marked for manual coding
        #region ValidateCrossFormRules

        public List<ProcessMessageDocument> ValidateCrossFormRules(IDS2019 report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOGEN402009(response, report);
            VRATOGEN438000(response, report);
            VRATOGEN438000_(response, report);
            VRATOGEN438000__(response, report);
            VRATOGEN438000___(response, report);
            VRATOGEN438000____(response, report);
            VRATOGEN438000_____(response, report);
            VRATOGEN438000______(response, report);
            VRATOGEN438000_______(response, report);
            VRATOGEN438000________(response, report);
            VRATOGEN438000_________(response, report);
            VRATOGEN438000__________(response, report);
            VRATOGEN438000___________(response, report);
            VRATOGEN438000____________(response, report);
            VRATOGEN438000_____________(response, report);
            VRATOGEN438000______________(response, report);
            VRATOGEN438000_______________(response, report);
            VRATOGEN438000________________(response, report);
            VRATOGEN438000_________________(response, report);
            VRATOGEN438000__________________(response, report);
            VRATOGEN438000___________________(response, report);
            VRATOGEN438000____________________(response, report);
            VRATOGEN438000_____________________(response, report);
            VRATOGEN438000______________________(response, report);
            VRATOGEN438000_______________________(response, report);
            VRATOGEN438000________________________(response, report);
            VRATOGEN438000_________________________(response, report);
            VRATOGEN438000__________________________(response, report);
            VRATOGEN438000___________________________(response, report);
            VRATOGEN438000____________________________(response, report);
            VRATOGEN438000_____________________________(response, report);
            VRATOGEN438000______________________________(response, report);
            VRATOGEN438000_______________________________(response, report);
            VRATOGEN438000________________________________(response, report);
            VRATOGEN438000_________________________________(response, report);
            VRATOGEN438000__________________________________(response, report);
            VRATOGEN438000___________________________________(response, report);
            VRATOGEN438000____________________________________(response, report);
            VRATOGEN438000_____________________________________(response, report);
            VRATOGEN438000______________________________________(response, report);
            VRATOGEN438000_______________________________________(response, report);
            VRATOGEN438000________________________________________(response, report);
            VRATOGEN438000_________________________________________(response, report);
            VRATOGEN438000__________________________________________(response, report);
            VRATOGEN438000___________________________________________(response, report);
            VRATOGEN438000____________________________________________(response, report);
            VRATOGEN438000_____________________________________________(response, report);
            VRATOGEN438000______________________________________________(response, report);
            VRATOGEN438000_______________________________________________(response, report);
            VRATOGEN438000________________________________________________(response, report);
            VRATOGEN438000_________________________________________________(response, report);
            VRATOGEN438000__________________________________________________(response, report);
            VRATOGEN438000___________________________________________________(response, report);
            VRATOGEN438000____________________________________________________(response, report);
            VRATOGEN438000_____________________________________________________(response, report);
            VRATOGEN438000______________________________________________________(response, report);
            VRATOGEN438000_______________________________________________________(response, report);
            VRATOGEN438000________________________________________________________(response, report);
            VRATOGEN438000_________________________________________________________(response, report);
            VRATOGEN438000__________________________________________________________(response, report);
            VRATOGEN438000___________________________________________________________(response, report);
            VRATOGEN438000____________________________________________________________(response, report);
            VRATOGEN438000_____________________________________________________________(response, report);
            VRATOGEN438000______________________________________________________________(response, report);
            VRATOGEN438000_______________________________________________________________(response, report);
            VRATOGEN438000________________________________________________________________(response, report);
            VRATOGEN438000_________________________________________________________________(response, report);
            VRATOGEN438000__________________________________________________________________(response, report);
            VRATOGEN438000___________________________________________________________________(response, report);
            VRATOGEN438000____________________________________________________________________(response, report);
            VRATOGEN438000_____________________________________________________________________(response, report);
            VRATOGEN438000______________________________________________________________________(response, report);
            VRATOGEN438000_______________________________________________________________________(response, report);
            VRATOGEN438000________________________________________________________________________(response, report);
            VRATOGEN438000_________________________________________________________________________(response, report);
            VRATOGEN438000__________________________________________________________________________(response, report);
            VRATOGEN438000___________________________________________________________________________(response, report);
            VRATOGEN438000____________________________________________________________________________(response, report);
            VRATOGEN438000_____________________________________________________________________________(response, report);
            VRATOGEN438001(response, report);
            VRATOGEN438001_(response, report);
            VRATOGEN438001__(response, report);
            VRATOGEN438001___(response, report);
            VRATOGEN438001____(response, report);
            VRATOGEN438001_____(response, report);
            VRATOGEN438001______(response, report);
            VRATOGEN438001_______(response, report);
            VRATOGEN438001________(response, report);
            VRATOGEN438001_________(response, report);
            VRATOGEN438001__________(response, report);
            VRATOGEN438001___________(response, report);
            VRATOGEN438001____________(response, report);
            VRATOGEN438001_____________(response, report);
            VRATOGEN438001______________(response, report);
            VRATOGEN438001_______________(response, report);
            VRATOGEN438001________________(response, report);
            VRATOGEN438001_________________(response, report);
            VRATOGEN438001__________________(response, report);
            VRATOGEN438001___________________(response, report);
            VRATOGEN438001____________________(response, report);
            VRATOGEN438001_____________________(response, report);
            VRATOGEN438001______________________(response, report);
            VRATOGEN438001_______________________(response, report);
            VRATOGEN438001________________________(response, report);
            VRATOGEN438001_________________________(response, report);
            VRATOGEN438001__________________________(response, report);
            VRATOGEN438001___________________________(response, report);
            VRATOGEN438001____________________________(response, report);
            VRATOGEN438001_____________________________(response, report);
            VRATOGEN438001______________________________(response, report);
            VRATOGEN438001_______________________________(response, report);
            VRATOGEN438001________________________________(response, report);
            VRATOGEN438001_________________________________(response, report);
            VRATOGEN438001__________________________________(response, report);
            VRATOGEN438001___________________________________(response, report);
            VRATOGEN438001____________________________________(response, report);
            VRATOGEN438001_____________________________________(response, report);
            VRATOGEN438001______________________________________(response, report);
            VRATOGEN438001_______________________________________(response, report);
            VRATOGEN438001________________________________________(response, report);
            VRATOGEN438001_________________________________________(response, report);
            VRATOGEN438001__________________________________________(response, report);
            VRATOGEN438001___________________________________________(response, report);
            VRATOGEN438001____________________________________________(response, report);
            VRATOGEN438001_____________________________________________(response, report);
            VRATOGEN438001______________________________________________(response, report);
            VRATOGEN438001_______________________________________________(response, report);
            VRATOGEN438001________________________________________________(response, report);
            VRATOGEN438001_________________________________________________(response, report);
            VRATOGEN438001__________________________________________________(response, report);
            VRATOGEN438001___________________________________________________(response, report);
            VRATOGEN438001____________________________________________________(response, report);
            VRATOGEN438001_____________________________________________________(response, report);
            VRATOGEN438001______________________________________________________(response, report);
            VRATOGEN438001_______________________________________________________(response, report);
            VRATOGEN438001________________________________________________________(response, report);
            VRATOGEN438001_________________________________________________________(response, report);
            VRATOGEN438001__________________________________________________________(response, report);
            VRATOGEN438001___________________________________________________________(response, report);
            VRATOGEN438001____________________________________________________________(response, report);
            VRATOGEN438001_____________________________________________________________(response, report);
            VRATOGEN438001______________________________________________________________(response, report);
            VRATOGEN438001_______________________________________________________________(response, report);
            VRATOGEN438001________________________________________________________________(response, report);
            VRATOGEN438001_________________________________________________________________(response, report);
            VRATOGEN438001__________________________________________________________________(response, report);
            VRATOGEN438001___________________________________________________________________(response, report);
            VRATOGEN438001____________________________________________________________________(response, report);
            VRATOGEN438001_____________________________________________________________________(response, report);
            VRATOGEN438001______________________________________________________________________(response, report);
            VRATOGEN438001_______________________________________________________________________(response, report);
            VRATOGEN438001________________________________________________________________________(response, report);
            VRATOGEN438001_________________________________________________________________________(response, report);
            VRATOGEN438001__________________________________________________________________________(response, report);
            VRATOGEN438001___________________________________________________________________________(response, report);
            VRATOGEN438001____________________________________________________________________________(response, report);
            VRATOGEN438001_____________________________________________________________________________(response, report);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
